/*-
 * ~~~~~~licensing~~~~~~
 * generator-api
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.base.generator.template.impl;

import java.io.IOException;
import java.net.URL;
import java.net.URLConnection;

import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.url.AbstractURLStreamHandlerService;
import org.osgi.service.url.URLStreamHandlerService;

import ru.entaxy.base.generator.template.Template;
import ru.entaxy.base.generator.template.TemplateService;

@Component(service = URLStreamHandlerService.class, property = "url.handler.protocol=templates", immediate = true)
public class TemplateServiceURLResolver extends AbstractURLStreamHandlerService {

	@Reference(cardinality = ReferenceCardinality.MANDATORY)
	TemplateService templateService;
	
	@Override
	public URLConnection openConnection(URL url) throws IOException {
		String templateId = url.toString();
		templateId = templateId.replace("templates:", "");
		templateId = templateId.replace("/", ".");
		
		Template template = templateService.getTemplateById(templateId);
		if (template == null) {
			// we'll try without file extension
			int index = templateId.lastIndexOf(".");
			if (index>0)
				templateId = templateId.substring(0, index);
			template = templateService.getTemplateById(templateId);
		}

		if (template == null)
			return null;
		
		String templateUrl = template.getTemplateLocation().toString() + template.getTemplateFullName();
		URL resultUrl = new URL(templateUrl);
		return resultUrl.openConnection();
	}

}
