/*-
 * ~~~~~~licensing~~~~~~
 * template-service-test
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.base.generator.template.shell;

import java.util.Comparator;
import java.util.stream.Collectors;

import org.apache.karaf.shell.api.action.Action;
import org.apache.karaf.shell.api.action.Command;
import org.apache.karaf.shell.api.action.lifecycle.Service;
import org.apache.karaf.shell.support.table.ShellTable;

import ru.entaxy.base.generator.template.Template;
import ru.entaxy.esb.platform.runtime.base.connecting.generator.Generator;
import ru.entaxy.esb.platform.runtime.base.connecting.generator.factory.GeneratorFactory;

@Service
@Command(scope = "templates", name = "list")
public class ListTemplates extends TemplateServiceSupport implements Action {

	@Override
	public Object execute() throws Exception {
		
		ShellTable table = new ShellTable();
		table.column("id");
		table.column("File");
		table.column("Location");
		table.column("Generator");
		
		for (Template t: templateService.getAllTemplates().stream().sorted(new Comparator<Template>() {

			@Override
			public int compare(Template o1, Template o2) {
				return o1.getId().compareToIgnoreCase(o2.getId());
			}
			
		}).collect(Collectors.toList()) ) {
			Generator g = (Generator)GeneratorFactory.createGenerator(t);
			table.addRow().addContent(
					t.getId(),
					t.getTemplateFullName(),
					t.getTemplateLocation()==null?"":t.getTemplateLocation().toString(), 
					(g==null?"NOT FOUND":g.getGeneratorId())
				);
		}
		
		table.print(System.out);
		
		return null;
	}

}
