/*-
 * ~~~~~~licensing~~~~~~
 * test-producers
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.base.objects.factory.configuration;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.gson.Gson;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;

import ru.entaxy.platform.base.objects.factory.EntaxyFactory;
import ru.entaxy.platform.base.objects.factory.EntaxyFactory.CONFIGURATION.FIELDS;

public class FieldsElement extends AbstractElement<FieldsElement> {
	
	private static final Logger log = LoggerFactory.getLogger(FieldsElement.class);
	
	public static final String ELEMENT_NAME = EntaxyFactory.CONFIGURATION.FIELDS_SECTION_NAME;

	protected Map<String, FieldElement> fields = new LinkedHashMap<>();
	
	@Override
	public FieldsElement fromJson(JsonElement jsonElement) {
		Gson gson = new Gson(); 
		if (jsonElement.isJsonObject()) {
			for (Entry<String, JsonElement> entry : jsonElement.getAsJsonObject().entrySet()) {
				
				Class<? extends FieldElement> feClass = FieldElement.class;
				
				if (entry.getValue().isJsonObject())
					if (entry.getValue().getAsJsonObject().has(FIELDS.ATTRIBUTES.IS_REF)) {
						JsonElement val = entry.getValue().getAsJsonObject().get(FIELDS.ATTRIBUTES.IS_REF);
						if (val.isJsonPrimitive() && val.getAsJsonPrimitive().isBoolean())
							if (val.getAsJsonPrimitive().getAsBoolean()) {
								feClass = RefFieldElement.class;
							}
					}
				
				FieldElement fe = gson.fromJson(entry.getValue(), feClass);
				fe.setName(entry.getKey());
				fe.setJsonOrigin(entry.getValue().isJsonObject()?entry.getValue().getAsJsonObject():new JsonObject());
				addField(fe);
			}
		}
		/*
		 * this.fields = gson.fromJson( jsonElement,
		 * TypeToken.getParameterized(ArrayList.class, FieldElement.class).getType() );
		 */
		// processOutputs();
		return this;
	}

	protected void addField(FieldElement fieldElement) {
		this.fields.put(fieldElement.getName(), fieldElement);
	}
	
	public Map<String, FieldElement> getFieldsMap(){
		return this.fields;
	}
	
	public List<FieldElement> getFields() {
		return new ArrayList<>(fields.values());
	}	
	
}
