/*-
 * ~~~~~~licensing~~~~~~
 * object-factory
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.base.objects.factory.impl;

import java.util.HashMap;
import java.util.Map;

import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;

import ru.entaxy.platform.base.objects.factory.EntaxyFactory;

@Component (service = FactoryRegistry.class, immediate = true)
public class FactoryRegistry {

	protected Map<String, String> inheritance = new HashMap<>();
	protected Map<String, EntaxyFactory> factories = new HashMap<>();
	
	@Reference (unbind = "removeFactory", cardinality = ReferenceCardinality.MULTIPLE, policy = ReferencePolicy.DYNAMIC
			, policyOption = ReferencePolicyOption.GREEDY)
	public void addFactory(EntaxyFactory entaxyFactory) {
		this.inheritance.put(entaxyFactory.getId(), entaxyFactory.getParent());
		this.factories.put(entaxyFactory.getId(), entaxyFactory);
	}
	
	public void removeFactory(EntaxyFactory entaxyFactory) {
		this.inheritance.remove(entaxyFactory.getId());
		this.factories.remove(entaxyFactory.getId());
	}
	
	public String getParentFactory(String factoryId) {
		return this.inheritance.get(factoryId);
	}
}
