/*-
 * ~~~~~~licensing~~~~~~
 * test-producers
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.base.objects.factory.tracker;

import java.util.List;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.service.component.ComponentContext;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Deactivate;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.util.tracker.BundleTracker;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.platform.base.support.osgi.tracker.BundleTrackerUtils;
import ru.entaxy.platform.base.support.osgi.tracker.filter.BundleHeaderFilter;

@Component(service = {TrackerManager.class}, immediate = true)
public class TrackerManager {

	private static final Logger log = LoggerFactory.getLogger(TrackerManager.class);
	
	protected BundleContext bundleContext;
	
	protected BundleTracker<List<TrackedFactory>> factoryTracker;
	
	@Reference (cardinality = ReferenceCardinality.MANDATORY)
	TrackedFactoryManager factoryManager;
	
	@Activate
	public void activate(ComponentContext componentContext) {
		this.bundleContext = componentContext.getBundleContext();
		log.debug("Activated: " + bundleContext.getBundle().getBundleId());
		
		factoryTracker = BundleTrackerUtils.<List<TrackedFactory>>createBuilder()
				.addFilter(
					(new BundleHeaderFilter()).header("Entaxy-Factory-Provider")	
				)
				.customizer(
					(new TrackedFactoryCustomizer())
					// .listener(new TrackedFactoryCustomizerListener(bundleContext))
					.listener(factoryManager)
				)
				.bundleState(Bundle.ACTIVE | Bundle.INSTALLED | Bundle.RESOLVED)
				.get();
		factoryTracker.open();
		log.debug("Factory tracker started");
	}
	
	@Deactivate
	public void deactivate(ComponentContext componentContext) {
		factoryTracker.close();
	}
}
