/*-
 * ~~~~~~licensing~~~~~~
 * resources-management
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.resources.management.impl;

import org.osgi.framework.BundleContext;
import org.osgi.service.component.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ru.entaxy.esb.platform.base.management.core.ManagementCore;
import ru.entaxy.esb.platform.base.management.core.api.AnnotatedMBean;
import ru.entaxy.esb.resources.EntaxyResourceService;
import ru.entaxy.esb.resources.management.EntaxyResourceServiceMBean;

import javax.management.DynamicMBean;
import javax.management.MBeanRegistration;
import javax.management.NotCompliantMBeanException;

@Component(service = { EntaxyResourceServiceMBean.class, DynamicMBean.class, MBeanRegistration.class }, property = {
        ManagementCore.JMX_OBJECTNAME + "=" + ManagementCore.Q_RUNTIME_S + "," + EntaxyResourceServiceMBean.RESOURCE_SERVICE_KEY + "="
                + EntaxyResourceServiceMBean.RESOURCE_SERVICE_KEY_VALUE }, scope = ServiceScope.SINGLETON, immediate = true)
public class EntaxyResourceServiceMBeanImpl extends AnnotatedMBean<EntaxyResourceServiceMBean>
        implements EntaxyResourceServiceMBean {

    private static final Logger log = LoggerFactory.getLogger(EntaxyResourceServiceMBeanImpl.class);

    protected BundleContext bundleContext;

    @Reference(cardinality = ReferenceCardinality.MANDATORY
            , service = EntaxyResourceService.class
            , collectionType = CollectionType.SERVICE)
    EntaxyResourceService entaxyResourceService;

    public EntaxyResourceServiceMBeanImpl() throws NotCompliantMBeanException {
        super(EntaxyResourceServiceMBean.class);
    }

    @Override
    public String getResource(String location) throws Exception {
        return entaxyResourceService.getResource(location).getAsString();
    }
}
