/*-
 * ~~~~~~licensing~~~~~~
 * resources-service
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.resources.impl;

import java.util.HashMap;
import java.util.Map;

import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicyOption;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.resources.EntaxyResource;
import ru.entaxy.esb.resources.EntaxyResourceProvider;
import ru.entaxy.esb.resources.EntaxyResourceService;
import ru.entaxy.platform.base.support.CommonUtils;

@Component (service = EntaxyResourceService.class, immediate = true)
public class EntaxyResourceServiceImpl implements EntaxyResourceService {

	private static final Logger log = LoggerFactory.getLogger(EntaxyResourceServiceImpl.class);
	
	protected Map<String, EntaxyResourceProvider> providers = new HashMap<>();
	
	@Reference (cardinality = ReferenceCardinality.MULTIPLE, collectionType = CollectionType.SERVICE
			, policyOption = ReferencePolicyOption.GREEDY
			, unbind = "removeProvider")
	public void addProvider(EntaxyResourceProvider provider) {
		synchronized (providers) {
			this.providers.put(provider.getProtocol(), provider);
			log.info("RESOURCE PROVIDER ADDED: " + provider.getProtocol());
		}
	}
	
	public void removeProvider(EntaxyResourceProvider provider) {
		synchronized (providers) {
			this.providers.remove(provider.getProtocol());
		}
	}
	
	
	@Override
	public EntaxyResource getResource(String location) {
		if (!CommonUtils.isValid(location))
			return null;
		int index = location.indexOf(":");
		if (index>0) {
			String protocol = location.substring(0, index);
			String path = location.substring(index+1);
			if (providers.containsKey(protocol)) {
				return providers.get(protocol).getResource(path);
			} else {
				log.warn("Provider not found for protocol: [{}]", protocol);
				return null;
			}
		}
		return null;
	}

}
