/*-
 * ~~~~~~licensing~~~~~~
 * resources-service
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.resources.tracker;

import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.osgi.framework.Bundle;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.platform.base.support.CommonUtils;

public class ResourceProvider {

	private static final Logger log = LoggerFactory.getLogger(ResourceProvider.class);
	
	public static final String RESOURCE_PATH = "/ru/entaxy/resource";
	
	protected Bundle bundle;
	
	protected Map<String, List<ResourceDescriptor>> resources = new HashMap<>();
	
	public ResourceProvider(Bundle bundle) {
		this.bundle = bundle;
	}
	
	public Map<String, List<ResourceDescriptor>> getResources(){
		if (!resources.isEmpty())
			return resources;
		
		Enumeration<URL> foundEntries = bundle.findEntries(ResourceProvider.RESOURCE_PATH, "*.*", true);
		while (foundEntries.hasMoreElements()) {
			URL entry = foundEntries.nextElement();
			log.debug("FOUND :: " + entry.toString());
			if (entry.toString().endsWith("/")) {
				log.debug(":: .. is folder");
				continue;
			}
			String localPath = entry.toString();
			localPath = localPath.substring(localPath.indexOf(ResourceProvider.RESOURCE_PATH) + ResourceProvider.RESOURCE_PATH.length() + 1);
			
			String resourceName = localPath.substring(localPath.lastIndexOf("/")+1);
			
			String path = localPath.lastIndexOf("/")>=0
					?localPath.substring(0, localPath.lastIndexOf("/"))
					:"";
			int splitIndex = path.indexOf("/");
			String protocol = splitIndex>0
					?path.substring(0, splitIndex)
					:"";
			
			if (splitIndex>0)
				path = path.substring(splitIndex+1);
			
			if (!CommonUtils.isValid(protocol) || !CommonUtils.isValid(resourceName))
				continue;
			
			if (!resources.containsKey(protocol))
				resources.put(protocol, new ArrayList<>());
			
			resources.get(protocol).add(new ResourceDescriptor(entry, path + "/" + resourceName));
			
		}
		
		return resources;
		
	}
	
	public static class ResourceDescriptor {
		
		public URL url;
		public String location;
		
		public ResourceDescriptor(URL url, String location) {
			this.url = url;
			this.location = location;
		}
		
	}
	
}
