/*-
 * ~~~~~~licensing~~~~~~
 * resources-service
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.resources.tracker;

import java.io.IOException;
import java.net.URL;
import java.util.List;
import java.util.Map;

import org.osgi.framework.BundleContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.resources.EntaxyResource;
import ru.entaxy.esb.resources.EntaxyResourceService;
import ru.entaxy.esb.resources.tracker.ResourceProvider.ResourceDescriptor;
import ru.entaxy.platform.base.support.osgi.tracker.BundleTrackerCustomizerListener;

public class ResourceProviderCustomizerListener implements BundleTrackerCustomizerListener<List<ResourceProvider>> {

	private static final Logger log = LoggerFactory.getLogger(ResourceProviderCustomizerListener.class);
	
	protected BundleContext bundleContext; 
	
	protected EntaxyResourceService entaxyResourceService; 
	
	public ResourceProviderCustomizerListener(BundleContext context, EntaxyResourceService service) {
		this.bundleContext = context;
		this.entaxyResourceService = service;
	}
	
	@Override
	public void added(List<ResourceProvider> managedObject) {
		for (ResourceProvider provider: managedObject) {
			Map<String, List<ResourceDescriptor>> resources = provider.getResources();
			String message = "";
			message += "\nFOUND RESOURCES";
			for (String protocol: resources.keySet()) {
				message += "\n\tPROTOCOL :: "  + protocol;
				List<ResourceDescriptor> list = resources.get(protocol);
				for (ResourceDescriptor rd: list) {
					message += "\n\t\t :: "  + rd.location + " | " + rd.url.toString();
				
					EntaxyResource resource = entaxyResourceService.getResource(protocol + ":" + rd.location);
					if (resource == null) {
						message += "\n\t\t -> NULL";
					} else {
						try {
							resource.save(rd.url.openStream());
							message += "\n\t\t -> " + resource.getAsString();
						} catch (IOException e) {
							log.error("Error saving resource", e);
						}
					}
						
				}
			}
			log.debug(message);
		}
		
	}

	@Override
	public void modified(List<ResourceProvider> managedObject) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void removed(List<ResourceProvider> managedObject) {
		// TODO Auto-generated method stub
		
	}

}
