/*-
 * ~~~~~~licensing~~~~~~
 * resources-service
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.resources.tracker;

import java.util.List;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.service.component.ComponentContext;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.util.tracker.BundleTracker;

import ru.entaxy.esb.resources.EntaxyResourceService;
import ru.entaxy.platform.base.support.osgi.tracker.BundleTrackerUtils;
import ru.entaxy.platform.base.support.osgi.tracker.filter.BundleHeaderFilter;

@Component
public class TrackerManager {

	protected BundleContext bundleContext;
	
	protected BundleTracker<List<ResourceProvider>> providerTracker;
	
	@Reference (cardinality = ReferenceCardinality.MANDATORY)
	EntaxyResourceService entaxyResourceService;
	
	@Activate
	public void activate(ComponentContext componentContext) {
		this.bundleContext = componentContext.getBundleContext();
		providerTracker = BundleTrackerUtils.<List<ResourceProvider>>createBuilder()
				.addFilter(
					(new BundleHeaderFilter()).header("Entaxy-Resource-Provider")	
				)
				.customizer(
					(new ResourceProviderCustomizer())
					.listener(new ResourceProviderCustomizerListener(bundleContext, entaxyResourceService))
				)
				.bundleState(Bundle.ACTIVE | Bundle.INSTALLED | Bundle.RESOLVED)
				.get();
		providerTracker.open();	}
	
}
