/*-
 * ~~~~~~licensing~~~~~~
 * schema-impl
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.runtime.core.infrastructure.schema.resolver.resource;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;

import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.url.AbstractURLStreamHandlerService;
import org.osgi.service.url.URLStreamHandlerService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.runtime.core.infrastructure.schema.api.ResourceService;
import ru.entaxy.esb.platform.runtime.core.infrastructure.schema.api.entity.Resource;

@Component(service = URLStreamHandlerService.class, property = "url.handler.protocol=resource-registry", immediate = true)
public class ResourceRegistryURLResolver extends AbstractURLStreamHandlerService {

	private static final Logger log = LoggerFactory.getLogger(ResourceRegistryURLResolver.class);
	
	@Reference (collectionType = CollectionType.SERVICE, cardinality = ReferenceCardinality.MANDATORY)
	ResourceService resourceService;
	
	protected static class SchemaRegistryURLConnection extends URLConnection {

		ResourceService resourceService;
		
		protected String resourceName = null;
		protected String resourceVersion = null;
		
		protected SchemaRegistryURLConnection(URL url, ResourceService resourceService) {
			super(url);
			this.resourceService = resourceService;
			log.info("NEW CONNECTION TO: " + url.toString());
			String remaining = url.toString().substring(url.toString().indexOf(":")+1);
			String[] splitted = remaining.split(File.separator);
			if (splitted.length>1) {
				resourceVersion = splitted[splitted.length - 1];
				resourceName = remaining.substring(0, remaining.length() - resourceVersion.length());
			} else {
				resourceName = remaining;
			}
		}

		@Override
		public void connect() throws IOException {
			log.info("CONNECTING: " + url.toString());
		}

		@Override
		public InputStream getInputStream() throws IOException {
			Resource resource = resourceService.getResourceByFullName(resourceName).orElse(null);
			log.info("RESOURCE [{}] {} FOUND "
					, resourceName
					, resource==null?"NOT":"");
			if (resource == null)
				return super.getInputStream();
			return new ByteArrayInputStream(resource.getResourceValue());
		}
		
	} 
	
	@Override
	public URLConnection openConnection(URL u) throws IOException {
		return new SchemaRegistryURLConnection(u, resourceService);
	}

}
