/*-
 * ~~~~~~licensing~~~~~~
 * test-producers
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.executor.commands;

import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.runtime.base.connecting.generator.Generated;
import ru.entaxy.platform.core.producer.api.EntaxyProducerService;
import ru.entaxy.platform.core.producer.api.ProducerResult;
import ru.entaxy.platform.core.producer.api.ProducerResult.CommandResult;
import ru.entaxy.platform.core.producer.executor.AbstractCommandExecutor;
import ru.entaxy.platform.core.producer.executor.CommandExecutor;
import ru.entaxy.platform.core.producer.executor.builder.BuiltObject;
import ru.entaxy.platform.core.producer.executor.builder.BuiltObjectList;
import ru.entaxy.platform.core.producer.executor.builder.ObjectBuilder;
import ru.entaxy.platform.core.producer.executor.generationmodel.GeneratedList;
import ru.entaxy.platform.core.producer.executor.support.ObjectSupportRegistry;
import ru.entaxy.platform.core.producer.executor.support.PrintOutputSupport;

@CommandExecutor(id = "build", predecessors = {"generate"})
public class Build extends AbstractCommandExecutor {

	private static final Logger log = LoggerFactory.getLogger(Build.class);
	
	public Build(EntaxyProducerService entaxyProducerService) {
		super(entaxyProducerService);
	}

	@Override
	protected boolean doExecute(ProducerResult currentResult, CommandResult commandResult,
			Map<String, Object> instructions) throws Exception {
		GeneratedList generated = currentResult.findResultObject(GeneratedList.class);
		if (generated == null) {
			log.info("Nothing to build");
			return true;
		}
		
		BuiltObjectList resultList = new BuiltObjectList();
		
		for (Generated g: generated) {
			if (g == null)
				continue;
			
			printOutput(" = Building for generated of type [" + g.getType() + "]" );
			
			ObjectBuilder builder = ObjectSupportRegistry.getInstance().findBuilder(g);
			if (builder == null) {
				log.warn("ObjectBuilder not found for generated object of type [{}]", g.getType());
				printOutput(" :: ObjectBuilder not found");
				continue;
			}
			
			printOutput(" :: ObjectBuilder is: " + builder.getClass().getName());
			BuiltObject result = builder.build(g, instructions);
			if (result != null) {
				resultList.add(result);
				printOutput(" :: Result is: " + result.getClass().getName());
			} else 
				printOutput(" :: Result is: NULL");
			
		}
		
		commandResult.resultObject(resultList);
		
		return true;
	}

}
