/*-
 * ~~~~~~licensing~~~~~~
 * test-producers
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.executor.commands;

import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.platform.core.producer.api.EntaxyProducerService;
import ru.entaxy.platform.core.producer.api.ProducerResult;
import ru.entaxy.platform.core.producer.api.ProducerResult.CommandResult;
import ru.entaxy.platform.core.producer.executor.AbstractCommandExecutor;
import ru.entaxy.platform.core.producer.executor.CommandExecutor;
import ru.entaxy.platform.core.producer.executor.builder.BuiltObject;
import ru.entaxy.platform.core.producer.executor.builder.BuiltObjectList;
import ru.entaxy.platform.core.producer.executor.deployer.DeployedObject;
import ru.entaxy.platform.core.producer.executor.deployer.DeployedObjectList;
import ru.entaxy.platform.core.producer.executor.deployer.ObjectDeployer;
import ru.entaxy.platform.core.producer.executor.support.ObjectSupportRegistry;

@CommandExecutor(id = "deploy", predecessors = {"build"})
public class Deploy extends AbstractCommandExecutor {

	private static final Logger log = LoggerFactory.getLogger(Deploy.class);
	
	public static final String DEPLOY_LOCAL_INSTRUCTION = "deployLocal";
	
	public Deploy(EntaxyProducerService entaxyProducerService) {
		super(entaxyProducerService);
	}

	@Override
	protected boolean doExecute(ProducerResult currentResult, CommandResult commandResult,
			Map<String, Object> instructions) throws Exception {

		BuiltObjectList objectList = currentResult.findResultObject(BuiltObjectList.class);
		if (objectList == null) {
			log.info("Nothing to deploy");
			return true;
		}		

		DeployedObjectList resultList = new DeployedObjectList();
		
		for (BuiltObject obj: objectList) {
			if (obj == null)
				continue;
			
			printOutput(" = Deploying for built of type [" + obj.getObject().getClass().getName() + "]" );
			
			ObjectDeployer deployer = ObjectSupportRegistry.getInstance().findDeployer(obj);
			if (deployer == null) {
				log.warn("ObjectDeployer not found for built object of type [{}]", obj.getObject().getClass().getName());
				printOutput(" :: ObjectDeployer not found");
				continue;
			}
			
			printOutput(" :: ObjectDeployer is: " + deployer.getClass().getName());
			DeployedObject result = deployer.deploy(obj, instructions);
			if (result != null) {
				resultList.add(result);
				printOutput(" :: Result is: " + result.getClass().getName());
			} else 
				printOutput(" :: Result is: NULL");
			
		}
		
		commandResult.resultObject(resultList);
		
		return true;
	}

}
