/*-
 * ~~~~~~licensing~~~~~~
 * test-producers
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.executor.commands;

import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.platform.core.producer.api.EntaxyProducerService;
import ru.entaxy.platform.core.producer.api.ProducerResult;
import ru.entaxy.platform.core.producer.api.ProducerResult.CommandResult;
import ru.entaxy.platform.core.producer.executor.AbstractCommandExecutor;
import ru.entaxy.platform.core.producer.executor.CommandExecutor;
import ru.entaxy.platform.core.producer.executor.deployer.DeployedObject;
import ru.entaxy.platform.core.producer.executor.deployer.DeployedObjectList;
import ru.entaxy.platform.core.producer.executor.installer.InstalledObject;
import ru.entaxy.platform.core.producer.executor.installer.InstalledObjectList;
import ru.entaxy.platform.core.producer.executor.installer.ObjectInstaller;
import ru.entaxy.platform.core.producer.executor.support.ObjectSupportRegistry;

@CommandExecutor(id = "install", predecessors = "deploy")
public class Install extends AbstractCommandExecutor {

	private static final Logger log = LoggerFactory.getLogger(Install.class);
	
	public static final String INSTALL_LOCAL_INSTRUCTION = "installLocal";
	public static final String UPDATE_INSTRUCTION = "update";
	public static final String INSTALL_ONLY_IF_MISSING_INSTRUCTION = "installOnlyIfMissing";

	public Install(EntaxyProducerService entaxyProducerService) {
		super(entaxyProducerService);
	}

	@Override
	protected boolean doExecute(ProducerResult currentResult, CommandResult commandResult,
			Map<String, Object> instructions) throws Exception {

		DeployedObjectList objectList = currentResult.findResultObject(DeployedObjectList.class);
		if (objectList == null) {
			log.info("Nothing to install");
			return true;
		}
		
		InstalledObjectList resultList = new InstalledObjectList();
		
		for (DeployedObject obj: objectList) {
			
			if (obj == null)
				continue;
			
			printOutput(" = Installing for deployed of type [" + obj.getObject().getClass().getName() + "]" );
			
			
			ObjectInstaller installer = ObjectSupportRegistry.getInstance().findInstaller(obj);
			if (installer == null) {
				log.warn("ObjectInstaller not found for deployed object of type [{}]", obj.getObject().getClass().getName());
				printOutput(" :: ObjectInstaller not found");
				continue;
			}
			
			printOutput(" :: ObjectInstaller is: " + installer.getClass().getName());
			InstalledObject result = installer.install(obj, instructions);
			if (result != null) {
				resultList.add(result);
				printOutput(" :: Result is: " + result.getClass().getName());
			} else 
				printOutput(" :: Result is: NULL");			
		}
		
		commandResult.resultObject(resultList);
		
		return true;
	}

}
