/*-
 * ~~~~~~licensing~~~~~~
 * test-producers
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.executor.commands;

import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.platform.core.producer.api.EntaxyProducerService;
import ru.entaxy.platform.core.producer.api.ProducerResult;
import ru.entaxy.platform.core.producer.api.ProducerResult.CommandResult;
import ru.entaxy.platform.core.producer.executor.AbstractCommandExecutor;
import ru.entaxy.platform.core.producer.executor.CommandExecutor;
import ru.entaxy.platform.core.producer.executor.objectmodel.ObjectModel;

@CommandExecutor(id = "layout", predecessors = {"analyze"})
public class Layout extends AbstractCommandExecutor {

	private static final Logger log = LoggerFactory.getLogger(Layout.class);
	
	public Layout(EntaxyProducerService entaxyProducerService) {
		super(entaxyProducerService);
	}

	@Override
	protected boolean doExecute(ProducerResult currentResult, CommandResult commandResult,
			Map<String, Object> instructions) throws Exception {
		log.debug("Executing command [{}]", getId());
		ObjectModel objectModel = currentResult.findResultObject(ObjectModel.class);
		
		printOutput("\n\tIncoming JSON ::\n\n");
		printOutput(objectModel.getJsonCurrent().toString());

		log.debug(" -- INCOMING JSON --\n" + objectModel.getJsonCurrent().toString());
		objectModel.layout();
		log.debug(" -- OUTGOING JSON --\n" + objectModel.getJsonCurrent().toString());
		commandResult.resultObject(objectModel);
		printOutput("\n\tOutgoing JSON ::\n\n");
		printOutput(objectModel.getJsonCurrent().toString());
		return true;
	}



}
