/*-
 * ~~~~~~licensing~~~~~~
 * object-producer-core
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.executor.generationmodel.merge;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import ru.entaxy.platform.core.producer.executor.generationmodel.merge.MergingMap.Mapping;

public abstract class AbstractMergeProcessor implements MergeProcessor {

	protected Map<String, List<String>> mapping;
	

	protected AbstractMergeProcessor() {
		super();
		mapping = new HashMap<>();
		if (this.getClass().isAnnotationPresent(MergingMap.class)) {
			MergingMap map = this.getClass().getAnnotation(MergingMap.class);
			Mapping[] mappings = map.mappings();
			if (mappings != null)
				for (int i=0; i<mappings.length; i++) {
					String source = mappings[i].source();
					String[] targets = mappings[i].targets();
					if ((targets == null) || (targets.length == 0))
						continue;
					this.mapping.put(source, Arrays.asList(targets));
				}
		}
	}
	
	@Override
	public Map<String, List<String>> getMergingMap() {
		return mapping;
	}

}
