/*-
 * ~~~~~~licensing~~~~~~
 * object-producer-core
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.executor.generationmodel.merge;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.osgi.service.component.ComponentContext;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;

import ru.entaxy.esb.platform.runtime.base.connecting.generator.Generated;

@Component(service = MergeHelper.class, immediate = true)
public class MergeHelper {

	private static MergeHelper _INSTANCE = null;
	
	public static MergeHelper getInstance() {
		return _INSTANCE;
	}
	
	protected static class MergeProcessorWrapper {
		MergeProcessor mergeProcessor;
		
		public MergeProcessorWrapper(MergeProcessor mergeProcessor) {
			this.mergeProcessor = mergeProcessor;
		}
	}
	
	protected Map<MergeProcessor, MergeProcessorWrapper> wrappersMap = new HashMap<>();
	
	protected Map<String, Map<String, MergeProcessorWrapper>> processors = new HashMap<>();
	
	@Activate
	public void activate(ComponentContext componentContext) {
		_INSTANCE = this;
	}
	
	@Reference(unbind = "removeMergeProcessor", cardinality = ReferenceCardinality.MULTIPLE, collectionType = CollectionType.SERVICE
			, policy = ReferencePolicy.DYNAMIC, policyOption = ReferencePolicyOption.GREEDY, service = MergeProcessor.class)
	public void addMergeProcessor(MergeProcessor mergeProcessor) {
		
		MergeProcessorWrapper wrapper = new MergeProcessorWrapper(mergeProcessor);
		
		synchronized (this.wrappersMap) {
			this.wrappersMap.put(mergeProcessor, wrapper);
		}

		Map<String, List<String>> mapping = mergeProcessor.getMergingMap();
		
		synchronized (this.processors) {
			for (Map.Entry<String, List<String>> entry: mapping.entrySet()) {
				this.processors.putIfAbsent(entry.getKey(), new HashMap<>());
				Map<String, MergeProcessorWrapper> map = this.processors.get(entry.getKey());
				for (String target: entry.getValue())
					map.put(target, wrapper);
			}
		}
		
	}

	public void removeMergeProcessor(MergeProcessor mergeProcessor) {
		synchronized (wrappersMap) {
			if (wrappersMap.containsKey(mergeProcessor)) {
				wrappersMap.get(mergeProcessor).mergeProcessor = null;
				wrappersMap.remove(mergeProcessor);
			}
		}
	}
	
	public boolean canMerge(String source, String target) {
		return processors.containsKey(source)
			&& processors.get(source).containsKey(target)
			&& processors.get(source).get(target).mergeProcessor != null;
	}
	
	public Generated merge(Generated source, Generated target) {
		if (!canMerge(source.getType(), target.getType()))
			return null;
		return processors.get(source.getType()).get(target.getType()).mergeProcessor.merge(source, target);
	}
}
