/*-
 * ~~~~~~licensing~~~~~~
 * object-producer-core
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.executor.objectmodel;

import java.util.LinkedList;
import java.util.Map;
import java.util.UUID;

import com.google.gson.JsonElement;
import com.google.gson.JsonObject;

import ru.entaxy.platform.base.support.JSONUtils.ObjectWrapper;

public abstract class AbstractItem extends ObjectWrapper {

	public static final String OBJECT_UUID = "__uuid__";
	
	public static final String FACTORED_CONTEXT_KEY = "factoredObjects";
	
	public String jsonPath = "";
	public String relativePath = "";
	public JsonObject origin;
	public FactoredObject definedIn = null;
	
	protected String uuid = null;
	
	protected ObjectModel objectModel;

	protected AbstractItem(ObjectModel objectModel) {
		this.objectModel = objectModel;
	}	
	
	public boolean isDefinedIn(FactoredObject fo) {
		if (fo == null)
			return false;
		FactoredObject test = this.definedIn;
		while (test!=null && test!=fo)
			test = test.definedIn;
		return test == fo;
	}

	protected void computeRelativePath(String parentPath) {
		this.relativePath = this.jsonPath.replace(parentPath, "");
		if (this.relativePath.startsWith("."))
			this.relativePath = this.relativePath.substring(1);
	}

	public static void initContext(Map<String, Object> context) {
		if (!context.containsKey(FACTORED_CONTEXT_KEY))
			context.put(FACTORED_CONTEXT_KEY, new LinkedList<>());
	}

	public void wrap(JsonObject object, Map<String, Object> context, String path) {
		this.origin = object;
		this.jsonPath = path;
		if (this.origin.has(OBJECT_UUID)) {
			JsonElement je = this.origin.get(OBJECT_UUID);
			if (je.isJsonPrimitive())
				if (je.getAsJsonPrimitive().isString())
					this.uuid = je.getAsJsonPrimitive().getAsString(); 
		}
		if (this.uuid == null) {
			this.uuid = UUID.randomUUID().toString();
			this.origin.addProperty(OBJECT_UUID, this.uuid);
		}
	}
	
	public String getUuid() {
		return uuid;
	}
	
	public void updateFromOrigin() {
		
	} 
}
