/*-
 * ~~~~~~licensing~~~~~~
 * object-producer-core
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.executor.support;

import java.util.List;

import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;

import ru.entaxy.esb.platform.runtime.base.connecting.generator.Generated;
import ru.entaxy.platform.core.producer.executor.builder.BuiltObject;
import ru.entaxy.platform.core.producer.executor.builder.ObjectBuilder;
import ru.entaxy.platform.core.producer.executor.deployer.DeployedObject;
import ru.entaxy.platform.core.producer.executor.deployer.ObjectDeployer;
import ru.entaxy.platform.core.producer.executor.installer.ObjectInstaller;

@Component (service = ObjectSupportRegistry.class, immediate = true)
public class ObjectSupportRegistry {

	private static ObjectSupportRegistry INSTANCE = null;
	
	public static ObjectSupportRegistry getInstance() {
		return INSTANCE;
	}
	
	@Reference (cardinality = ReferenceCardinality.MULTIPLE, policy = ReferencePolicy.DYNAMIC, policyOption = ReferencePolicyOption.GREEDY)
	volatile List<ObjectBuilder> objectBuilders;

	@Reference (cardinality = ReferenceCardinality.MULTIPLE, policy = ReferencePolicy.DYNAMIC, policyOption = ReferencePolicyOption.GREEDY)
	volatile List<ObjectDeployer> objectDeployers;
	
	@Reference (cardinality = ReferenceCardinality.MULTIPLE, policy = ReferencePolicy.DYNAMIC, policyOption = ReferencePolicyOption.GREEDY)
	volatile List<ObjectInstaller> objectInstallers;
	
	@Activate
	public void activate() {
		INSTANCE = this;
	}
	
	public ObjectBuilder findBuilder(Generated generated) {
		synchronized (this.objectBuilders) {
			for (ObjectBuilder ob: this.objectBuilders)
				if (ob.isBuildable(generated))
					return ob;
		}
		return null;
	}

	public ObjectDeployer findDeployer(BuiltObject object) {
		synchronized (this.objectDeployers) {
			for (ObjectDeployer od: this.objectDeployers)
				if (od.isDeployable(object))
					return od;
		}
		return null;
	}
	
	public ObjectInstaller findInstaller(DeployedObject object) {
		synchronized (this.objectInstallers) {
			for (ObjectInstaller oi: this.objectInstallers)
				if (oi.isInstallable(object))
					return oi;
		}
		return null;
	}
	
}
