/*-
 * ~~~~~~licensing~~~~~~
 * object-producer-core
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.impl;

import java.util.List;

import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.FieldOption;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;

import ru.entaxy.platform.core.producer.api.EntaxyFactoryDataProcessor;

@Component (service = EntaxyFactoryDataProcessorService.class, immediate = true)
public class EntaxyFactoryDataProcessorService implements EntaxyFactoryDataProcessor {

	private static EntaxyFactoryDataProcessorService _INSTANCE = null;
	
	public static EntaxyFactoryDataProcessorService getInstance() {
		return _INSTANCE;
	}
	
	@Reference (cardinality = ReferenceCardinality.MULTIPLE, collectionType = CollectionType.SERVICE, fieldOption = FieldOption.UPDATE, 
			policy = ReferencePolicy.DYNAMIC, policyOption = ReferencePolicyOption.GREEDY, service = EntaxyFactoryDataProcessor.class)
	List<EntaxyFactoryDataProcessor> processors;
	
	@Activate
	public void activate() {
		_INSTANCE = this;
	}
	
	@Override
	public Object processDefaultValue(Object currentValue) {
		Object result = currentValue;
		for (EntaxyFactoryDataProcessor p: processors) {
			result = p.processDefaultValue(result);
		}
		return result;
	}
	
}
