/*-
 * ~~~~~~licensing~~~~~~
 * object-producer-core
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;

import ru.entaxy.platform.base.objects.factory.EntaxyFactory.CONFIGURATION.DIRECTIVES.OVERRIDE_MODE;
import ru.entaxy.platform.base.objects.factory.EntaxyFactoryUtils;
import ru.entaxy.platform.base.support.JSONUtils;
import ru.entaxy.platform.core.producer.api.EntaxyProducerService;

public class InstructionsHelper {

	public static final String DEFAULT_INSTRUCTIONS = "{'validate':{}}";
	public static final String LIFECYCLE_DIRECTIVE = EntaxyProducerService.DIRECTIVES.LIFECYCLE;
	
	protected JsonObject lifecyclesOrigin;
	
	protected Map<String, JsonObject> lifecycles = new HashMap<>();
	
	public InstructionsHelper(JsonObject lifecycles) {
		super();
		this.lifecyclesOrigin = lifecycles.deepCopy();
		fillMap(this.lifecyclesOrigin);
	}
	
	protected void fillMap(JsonObject origin) {
		for (Entry<String, JsonElement> entry: origin.entrySet())
			if (entry.getValue().isJsonObject())
				this.lifecycles.put(entry.getKey(), entry.getValue().getAsJsonObject().deepCopy());
	}
	
	public String prepareInstructions(String originalInstructions) {
		JsonObject origin = JSONUtils.getJsonRootObject(originalInstructions);
		JsonObject result = new JsonObject();
		if (origin.has(LIFECYCLE_DIRECTIVE)) {
			JsonElement directiveValue = origin.get(LIFECYCLE_DIRECTIVE);
			List<String> lcIdToApply = new ArrayList<>();
			List<JsonObject> lcToApply = new ArrayList<>();
			if (directiveValue.isJsonPrimitive())
				lcIdToApply.add(directiveValue.getAsString());
			if (directiveValue.isJsonArray()) {
				JsonArray ja = directiveValue.getAsJsonArray();
				for (int i=0; i<ja.size(); i++)
					lcIdToApply.add(ja.get(i).getAsString());
			}
			origin.remove(LIFECYCLE_DIRECTIVE);
			for (String id: lcIdToApply)
				if (this.lifecycles.containsKey(id))
					EntaxyFactoryUtils.processObjectOverriding(
							result
							, this.lifecycles.get(id)
							, OVERRIDE_MODE.UPDATE);
		}
		EntaxyFactoryUtils.processObjectOverriding(
				result
				, origin
				, OVERRIDE_MODE.UPDATE);
		return result.toString();
	}
	
}
