/*-
 * ~~~~~~licensing~~~~~~
 * object-producing-config-support
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.config;

import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;

import ru.entaxy.platform.base.objects.EntaxyObject;
import ru.entaxy.platform.base.objects.EntaxyObjectService;
import ru.entaxy.platform.core.producer.executor.objectmodel.FactoredObjectRef;

@Component(service = ObjectConfigHelper.class, immediate = true)
public class ObjectConfigHelper {

	private static ObjectConfigHelper _INSTANCE = null;
	
	public static ObjectConfigHelper getInstance() {
		return _INSTANCE;
	}
	
	@Reference(cardinality = ReferenceCardinality.MANDATORY)
	EntaxyObjectService entaxyObjectService;
	
	public ObjectConfigHelper() {
		super();
		_INSTANCE = this;
	}
	
	public EntaxyObject findObject(String objectId, String objectType) {
		return entaxyObjectService.findObject(objectId, objectType);
	}
	
	public boolean isDirectValueOnly(FactoredObjectRef ref) {
		if (!ref.origin.has(ObjectConfig.DIRECT_VALUE_ONLY_ATTR_NAME))
			return false;
		try {
			return ref.origin.get(ObjectConfig.DIRECT_VALUE_ONLY_ATTR_NAME).getAsBoolean();
		} catch (Exception e) {
			return false;
		}
	}
	
}
