/*-
 * ~~~~~~licensing~~~~~~
 * object-producing-management
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.management.impl;

import java.util.HashMap;
import java.util.Map;

import javax.management.DynamicMBean;
import javax.management.MBeanRegistration;
import javax.management.NotCompliantMBeanException;

import org.osgi.framework.BundleContext;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;
import org.osgi.service.component.annotations.ServiceScope;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.base.management.core.ManagementCore;
import ru.entaxy.esb.platform.base.management.core.api.AnnotatedMBean;
import ru.entaxy.platform.base.objects.factory.EntaxyFactory;
import ru.entaxy.platform.core.producer.api.EntaxyProducer;
import ru.entaxy.platform.core.producer.management.FactoriesMBean;
import ru.entaxy.platform.core.producer.management.ProducerMBean;

@Component(service = { FactoriesMBean.class, DynamicMBean.class, MBeanRegistration.class }, property = {
		ManagementCore.JMX_OBJECTNAME + "=" + ManagementCore.Q_PLATFORM_S + "," + FactoriesMBean.FACTORIES_KEY + "="
				+ FactoriesMBean.FACTORIES_KEY_VALUE }, scope = ServiceScope.SINGLETON, immediate = true)
public class FactoriesMBeanImpl extends AnnotatedMBean<FactoriesMBean> implements FactoriesMBean {

	private static final Logger log = LoggerFactory.getLogger(FactoriesMBeanImpl.class);
	
	protected Map<EntaxyProducer, ManagedTypedProducers> producersMap = new HashMap<>(); 
	
	
	protected BundleContext bundleContext;
	
	protected TypedProducerListener producerListener = null;
	
	// to ensure FactoryMBeanPublisher is started 
	@Reference(cardinality = ReferenceCardinality.MANDATORY)
	FactoryMBeanPublisher factoryMBeanPublisher;
	
	@Activate
	public FactoriesMBeanImpl(BundleContext bundleContext) throws NotCompliantMBeanException {
		super(FactoriesMBean.class);
		this.bundleContext = bundleContext;
		log.debug("\n\tCONSTRUCTOR, bundleContext: [{}]\n"
				, this.bundleContext==null?"null":this.bundleContext.getBundle().getBundleId());
		this.producerListener = new TypedProducerListener(bundleContext, ProducerMBean.class);
	}

	
	@Reference(bind="bindProducer", unbind = "unbindProducer", cardinality = ReferenceCardinality.MULTIPLE
			, policy = ReferencePolicy.DYNAMIC
			, policyOption = ReferencePolicyOption.GREEDY, collectionType = CollectionType.SERVICE
			, service = EntaxyProducer.class)
	public void bindProducer(EntaxyProducer producer) {
		log.debug("\n\tBinding producer: [{}]\n", producer.getProducerId());
		synchronized (producersMap) {
			if (producersMap.containsKey(producer)) {
				producerListener.removed(producersMap.get(producer));
			}
			ManagedTypedProducers mtp = new ManagedTypedProducers(producer);
			producersMap.put(producer, mtp);
			if (producerListener != null)
				producerListener.added(mtp);
		}
	}

	public void unbindProducer(EntaxyProducer producer) {
		synchronized (producersMap) {
			if (producersMap.containsKey(producer)) {
				if (producerListener != null)
					producerListener.removed(producersMap.get(producer));
				producersMap.remove(producer);
			}
		}
	}
	
	public void bindFactory(EntaxyFactory entaxyFactory) {
		
	}
	
}
