/*-
 * ~~~~~~licensing~~~~~~
 * object-producing-resources-support
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.resources;

import java.util.Map;

import com.google.gson.JsonElement;
import com.google.gson.JsonObject;

import ru.entaxy.platform.base.support.CommonUtils;

public class ResourceDescriptor {

	static public final String RESOURCE_TYPE_PREFIX = "resource:";
	
	public static ResourceDescriptor fromJson(JsonElement element) {
		if (element == null)
			return null;
		if (!element.isJsonObject())
			return null;
		return new ResourceDescriptor(element.getAsJsonObject());
	}

	public static ResourceDescriptor fromMap(Map<String, Object> properties) {
		return new ResourceDescriptor(properties);
	}
	
	
	protected JsonObject origin;
	
	public String provider;
	public String location;
	public String endType;
	public String format;
	
	public ResourceDescriptor(JsonObject jsonObject) {
		this.origin = jsonObject;
		loadData();
	}

	public ResourceDescriptor(Map<String, Object> properties) {
		this.origin = new JsonObject();
		loadDataFromMap(properties);
	}
	
	protected void loadData() {
		if (this.origin.has(ResourceDataProcessor.PROP_PROVIDER))
			this.provider = this.origin.get(ResourceDataProcessor.PROP_PROVIDER).getAsString();
		if (this.origin.has(ResourceDataProcessor.PROP_LOCATION))
			if (this.origin.get(ResourceDataProcessor.PROP_LOCATION).isJsonPrimitive())
				this.location = this.origin.get(ResourceDataProcessor.PROP_LOCATION).getAsString();
		if (this.origin.has(ResourceDataProcessor.PROP_END_TYPE))
			this.endType = this.origin.get(ResourceDataProcessor.PROP_END_TYPE).getAsString();
		if (this.origin.has(ResourceDataProcessor.PROP_FORMAT))
			this.format = this.origin.get(ResourceDataProcessor.PROP_FORMAT).getAsString();
	}
	
	protected void loadDataFromMap(Map<String, Object> properties) {
		this.provider = properties.getOrDefault(ResourceDataProcessor.PROP_PROVIDER, "default").toString();
		this.location = properties.getOrDefault(ResourceDataProcessor.PROP_LOCATION, "").toString();
		this.endType = properties.getOrDefault(ResourceDataProcessor.PROP_END_TYPE, "").toString();
		this.format = properties.getOrDefault(ResourceDataProcessor.PROP_FORMAT, "").toString();
	};
	
	public String getResourceUrl() {
		return (CommonUtils.isValid(provider)?provider+":":"") + location;
	}
	
	public boolean isReturnResource() {
		return !CommonUtils.isValid(endType) || "resource".equalsIgnoreCase(endType);
	}
	
}
