/*-
 * ~~~~~~licensing~~~~~~
 * object-producing-resources-support
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.resources;

import java.io.InputStream;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.io.IOUtils;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;

import ru.entaxy.esb.platform.runtime.base.connecting.generator.Generated;
import ru.entaxy.esb.resources.EntaxyResource;
import ru.entaxy.esb.resources.EntaxyResourceService;
import ru.entaxy.platform.core.producer.executor.builder.BuiltObject;
import ru.entaxy.platform.core.producer.executor.builder.DefaultBuiltObject;
import ru.entaxy.platform.core.producer.executor.builder.ObjectBuilder;
import ru.entaxy.platform.core.producer.executor.deployer.DeployedObject;
import ru.entaxy.platform.core.producer.executor.deployer.ObjectDeployer;
import ru.entaxy.platform.core.producer.executor.support.LocalPrintOutput;

@Component(service = {ObjectBuilder.class, ObjectDeployer.class})
public class ResourceObjectSupport implements ObjectBuilder, ObjectDeployer {

	@Reference (cardinality = ReferenceCardinality.MANDATORY)
	EntaxyResourceService entaxyResourceService;
	
	/*
	 * ObjectBuilder
	 */
	@Override
	public boolean isBuildable(Generated generated) {
		return generated.getType().startsWith(ResourceDescriptor.RESOURCE_TYPE_PREFIX);
	}

	@Override
	public BuiltObject build(Generated generated, Map<String, Object> instructions) throws Exception {
		if (!isBuildable(generated))
			return null;
		
		LocalPrintOutput printer = new LocalPrintOutput(instructions);
		printer.printOutput(" = GENERATED PROPERTIES");
		for (Entry<String, Object> entry: generated.getProperties().entrySet()) {
			printer.printOutput(" :: " + entry.getKey() + " = [\n"
					+ (entry.getValue()==null?"":entry.getValue().toString())
					+ "\n]");
		}
		printer.printOutput(" = /GENERATED PROPERTIES");
		return new DefaultBuiltObject(new DeployableResource(generated.getProperties()));
		
	}

	/*
	 * ObjectDeployer 
	 */
	
	@Override
	public boolean isDeployable(BuiltObject object) {
		return object.getObject() instanceof DeployableResource;
	}

	@Override
	public DeployedObject deploy(BuiltObject object, Map<String, Object> instructions) throws Exception {
		if (!isDeployable(object))
			return null;
		LocalPrintOutput printer = new LocalPrintOutput(instructions);
		
		DeployableResource dr = (DeployableResource)object.getObject();
		printer.printOutput(" = RESOURCE TO DEPLOY");
		printer.printOutput(dr.getResourceUrl() + " [" + dr.resourceType + "/" + dr.endType + "]");
		
		EntaxyResource resource = entaxyResourceService.getResource(dr.getResourceUrl());
		resource.save(getInputStream(dr));
		
		printer.printOutput(" :: DEPLOYED");
		
		// nothing to do with resource in further phases
		return null;
	}

	protected InputStream getInputStream(DeployableResource res) {
		if ((res==null) || (res.resourceData == null))
			return getDefaultInputStream();
		if ("String".equals(res.endType))
			return IOUtils.toInputStream(res.resourceData.toString());
		// TODO support other resource types
		return getDefaultInputStream();
			
	}

	protected InputStream getDefaultInputStream() {
		return IOUtils.toInputStream("");
	}
}
