/*-
 * ~~~~~~licensing~~~~~~
 * object-producing-resources-support
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.resources.generator;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.ArrayUtils;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Deactivate;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;

import ru.entaxy.esb.platform.runtime.base.connecting.generator.AbstractSelfPublishGenerator;
import ru.entaxy.esb.platform.runtime.base.connecting.generator.Generated;
import ru.entaxy.esb.platform.runtime.base.connecting.generator.Generator;
import ru.entaxy.esb.platform.runtime.base.connecting.generator.GeneratorService;
import ru.entaxy.esb.platform.runtime.base.connecting.generator.factory.GeneratorFactory;
import ru.entaxy.esb.resources.EntaxyResource;
import ru.entaxy.esb.resources.EntaxyResourceService;
import ru.entaxy.platform.core.producer.resources.EntaxyResourceProcessorService;
import ru.entaxy.platform.core.producer.resources.ResourceDataProcessor;
import ru.entaxy.platform.core.producer.resources.ResourceDescriptor;
import ru.entaxy.platform.core.producer.resources.ResourceProducer;
import ru.entaxy.platform.core.producer.resources.StringFormatterService;

@Component(
		service = {GeneratorService.class},
		property = {GeneratorService.PROP_GENERATOR_TYPE + "=" + ResourceWrapGenerator.GENERATOR_NAME},
		immediate = true
)
public class ResourceWrapGenerator extends AbstractSelfPublishGenerator<ResourceWrapGenerator> 
		implements Generator {

	public static final String GENERATOR_NAME = "resource-wrap";

	protected static EntaxyResourceService ENTAXY_RESOURCE_SERVICE;

	protected static StringFormatterService STRING_FORMATTER_SERVICE;
	
	protected static EntaxyResourceProcessorService RESOURCE_PROCESSOR_SERVICE;
	
	@Reference (cardinality = ReferenceCardinality.MANDATORY)
	EntaxyResourceService entaxyResourceService;
	
	@Reference (cardinality = ReferenceCardinality.MANDATORY)
	StringFormatterService stringFormatterService;

	@Reference (cardinality = ReferenceCardinality.MANDATORY)
	EntaxyResourceProcessorService entaxyResourceProcessorService;
	
    @Activate
    public void activate() {
    	GeneratorFactory.register(ResourceWrapGenerator.GENERATOR_NAME, this);
    	ResourceWrapGenerator.ENTAXY_RESOURCE_SERVICE = entaxyResourceService;
    	ResourceWrapGenerator.STRING_FORMATTER_SERVICE = stringFormatterService;
    	ResourceWrapGenerator.RESOURCE_PROCESSOR_SERVICE = entaxyResourceProcessorService;
    }
    
    @Deactivate
    public void deactivate() {
    	GeneratorFactory.unregister(GENERATOR_NAME);
    }
	
	@Override
	public Generated generate(Map<String, Object> properties) throws Exception {
		ResourceDescriptor descriptor = ResourceDescriptor.fromMap(properties);
		String content = null;
		String object = null;
		if (properties.containsKey(ResourceProducer.PROP_CONTENT)) {
			content = properties.get(ResourceProducer.PROP_CONTENT).toString();
			String[] formatters = descriptor.format.split(",");
			ArrayUtils.reverse(formatters);
			object = STRING_FORMATTER_SERVICE.unformat(content, formatters);
			if (properties.containsKey(ResourceProducer.PROP_PREPROCESS)) {
				Object pp = properties.get(ResourceProducer.PROP_PREPROCESS);
				if (pp instanceof List) {
					List list = (List)pp;
					for (Object obj: list) {
						if (obj instanceof Map) {
							Object result = RESOURCE_PROCESSOR_SERVICE.process(object, (Map<String, Object>)obj);
							if (result != null)
								object = result.toString();
							else
								object = "";
						}
					}
				}
			}
		} else {
			EntaxyResource resource = ENTAXY_RESOURCE_SERVICE.getResource(descriptor.getResourceUrl());
			if (resource.exists()) {
				object = resource.getAsString();
				content = STRING_FORMATTER_SERVICE.format(object, descriptor.format.split(","));
			} 
		}
		if (object == null)
			return null;
		Map<String, Object> resultMap = new HashMap<>();
		resultMap.putAll(properties);
		resultMap.put(ResourceProducer.PROP_CONTENT, content);
		resultMap.put("#_object", object);
		return new GeneratedResourceWrapper(resultMap);
	}

	@Override
	public boolean isGeneratable(Map<String, Object> properties) throws Exception {
		return properties.containsKey(ResourceDataProcessor.PROP_PROVIDER)
			&& properties.containsKey(ResourceDataProcessor.PROP_LOCATION);
	}
	
}
