/*-
 * ~~~~~~licensing~~~~~~
 * object-producing-resources-support
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.resources.impl;

import java.util.HashMap;
import java.util.Map;

import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.platform.core.producer.resources.EntaxyResourceProcessor;
import ru.entaxy.platform.core.producer.resources.EntaxyResourceProcessorService;

@Component (service = EntaxyResourceProcessorService.class, immediate = true)
public class EntaxyResourceProcessorServiceImpl implements EntaxyResourceProcessorService {

	private static final Logger log = LoggerFactory.getLogger(EntaxyResourceProcessorServiceImpl.class);
	
	protected static EntaxyResourceProcessorService _INSTANCE;
	
	public static EntaxyResourceProcessorService getInstance() {
		return _INSTANCE;
	}
	
	protected Map<String, EntaxyResourceProcessor> processorsMap = new HashMap<>(); 
	
	@Activate
	public void activate() {
		_INSTANCE = this;
	}
	
	@Reference (cardinality = ReferenceCardinality.MULTIPLE, collectionType = CollectionType.SERVICE, policy = ReferencePolicy.DYNAMIC
			, policyOption = ReferencePolicyOption.GREEDY, unbind = "removeProcessor")
	public void addProcessor(EntaxyResourceProcessor processor) {
		synchronized (processorsMap) {
			this.processorsMap.put(processor.getName(), processor);
		}
	}

	public void removeProcessor(EntaxyResourceProcessor processor) {
		synchronized (processorsMap) {
			this.processorsMap.remove(processor.getName());
		}
	}
	
	
	@Override
	public Object process(Object value, Map<String, Object> properties) {
		Object val = properties.get(EntaxyResourceProcessor.PROP_PROCESSOR);
		if (val == null)
			return value;
		String processorName = val.toString();
		if (!processorsMap.containsKey(processorName)) {
			log.warn("Unknown processor [{}] in [{}]", processorName, properties);
			return value;
		}
		return processorsMap.get(processorName).process(value, properties);
	}

}
