/*-
 * ~~~~~~licensing~~~~~~
 * object-producing-resources-support
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.resources.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;

import ru.entaxy.platform.core.producer.resources.EntaxyResourceStringFormatter;
import ru.entaxy.platform.core.producer.resources.StringFormatterService;

@Component (service = StringFormatterService.class, immediate = true)
public class StringFormatterServiceImpl implements StringFormatterService {

	protected static StringFormatterService _INSTANCE;
		
	public static StringFormatterService getInstance() {
		return _INSTANCE;
	}
	
	protected Map<String, EntaxyResourceStringFormatter> formattersMap = new HashMap<>(); 
	
	@Activate
	public void activate() {
		_INSTANCE = this;
	}
	
	@Reference (cardinality = ReferenceCardinality.MULTIPLE, collectionType = CollectionType.SERVICE, policy = ReferencePolicy.DYNAMIC
			, policyOption = ReferencePolicyOption.GREEDY, unbind = "removeFormatter")
	public void addFormatter(EntaxyResourceStringFormatter formatter) {
		synchronized (formattersMap) {
			this.formattersMap.put(formatter.getName(), formatter);
		}
	}

	public void removeFormatter(EntaxyResourceStringFormatter formatter) {
		synchronized (formattersMap) {
			this.formattersMap.remove(formatter.getName());
		}
	}
	
	@Override
	public String format(String value, String[] formatters) {
		String current = value;
		for (int i=0; i<formatters.length; i++) {
			String formatter = formatters[i].trim();
			if (this.formattersMap.containsKey(formatter))
				current = this.formattersMap.get(formatter).format(current);
		}
		return current;
	}

	@Override
	public String unformat(String value, String[] formatters) {
		String current = value;
		for (int i=0; i<formatters.length; i++) {
			String formatter = formatters[i].trim();
			if (this.formattersMap.containsKey(formatter))
				current = this.formattersMap.get(formatter).unformat(current);
		}
		return current;
	}
}
