/*-
 * ~~~~~~licensing~~~~~~
 * test-producers
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.shell;

import java.util.List;

import org.apache.karaf.shell.api.action.Command;
import org.apache.karaf.shell.api.action.Option;
import org.apache.karaf.shell.api.action.lifecycle.Service;
import org.apache.karaf.shell.support.table.ShellTable;

import ru.entaxy.platform.base.objects.factory.EntaxyFactory;
import ru.entaxy.platform.base.objects.factory.EntaxyFactory.FieldInfo;
import ru.entaxy.platform.base.objects.factory.EntaxyFactory.RefFieldInfo;
import ru.entaxy.platform.base.support.CommonUtils;

@Service
@Command(scope = EntaxyProducerServiceSupport.SCOPE, name = "factory-fields")
public class FactoryFields extends FactoryAwareCommand {

	@Option(name = "-o", aliases = {"--output"}, required = false)
	String outputType;
	
	@Override
	protected Object doExecute() throws Exception {
		
		ShellTable table = new ShellTable();
		table.column("Name");
		table.column("Type");
		table.column("DisplayName");
		table.column("DefaultValue").maxSize(50).wrap(false);
		table.column("Required");
		table.column("Immutable");
		table.column("Is ref");
		table.column("Is back ref");
		table.column("Is ref by value");
		
		List<FieldInfo> fields;
		if (CommonUtils.isValid(outputType)) {
			System.out.println("FOR outputType = " + outputType);
			fields = entaxyFactory.getFields(outputType);
		} else {
			System.out.println("FOR default outputType = ");
			fields = entaxyFactory.getFields();
		}
		
		for (EntaxyFactory.FieldInfo fi: fields)
			table.addRow().addContent(
					fi.getName(),
					fi.getType(),
					fi.getDisplayName(),
					fi.getDefaultValue(),
					(fi.isRequired()?"*":""),
					(fi.isImmutable()?"*":""),
					(fi.isRef()?"*":""),
					(fi.isRef()
							?(((RefFieldInfo)fi).isBackRef()
									?"*"
									:"")
							:""),
					(fi.isRef()
							?(((RefFieldInfo)fi).isRefByValueOnly()
									?"*"
									:"")
							:"")
				);
		
		table.print(System.out);
		
		return null;
	}

}
