/*-
 * ~~~~~~licensing~~~~~~
 * connector-producing
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.base.objects.connector.producing;

import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;

import ru.entaxy.platform.base.objects.EntaxyObject.OBJECT_TYPES;
import ru.entaxy.platform.base.objects.factory.EntaxyFactory;
import ru.entaxy.platform.core.producer.api.EntaxyProducer;
import ru.entaxy.platform.core.producer.impl.CommonObjectProducer;
import ru.entaxy.platform.core.producer.impl.EntaxyProducerInfo;
import ru.entaxy.platform.core.producer.wrapper.AbstractFactoryWrapper;
import ru.entaxy.platform.core.producer.wrapper.DefaultFactoryWrapper;

@Component(service = {EntaxyProducer.class}, immediate = true)
@EntaxyProducerInfo(supportedTypes = {OBJECT_TYPES.CONNECTOR})
public class ConnectorProducer extends CommonObjectProducer implements EntaxyProducer {

	@Reference(bind = "addFactory", unbind = "removeFactory", cardinality = ReferenceCardinality.MULTIPLE
			, collectionType = CollectionType.SERVICE, policy = ReferencePolicy.DYNAMIC, policyOption = ReferencePolicyOption.GREEDY)
	public void addFactory(EntaxyFactory factory) {
		doAddFactory(factory);
	}
	
	// WE MUST DECLARE IT for @Reference annotation processor
	@Override
	public void removeFactory(EntaxyFactory factory) {
		super.removeFactory(factory);
	}

	@Override
	protected AbstractFactoryWrapper doAddFactory(EntaxyFactory factory) {
		AbstractFactoryWrapper wrapper = super.doAddFactory(factory, DefaultFactoryWrapper.class);
		if (wrapper != null)
			wrapper.setGenerationProcessor(this);
		return wrapper;
	}
	
}
