/*-
 * ~~~~~~licensing~~~~~~
 * connector-storage
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.base.objects.connector.storage;

import com.google.gson.Gson;
import com.google.gson.JsonObject;
import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ru.entaxy.esb.system.connector.impl.ConnectorService;
import ru.entaxy.esb.system.jpa.SystemService;
import ru.entaxy.esb.system.management.bundle.jpa.entity.BundleEntity;
import ru.entaxy.platform.base.objects.EntaxyObject;
import ru.entaxy.platform.base.objects.EntaxyObjectStorage;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

@Component(service = EntaxyObjectStorage.class, immediate = true)
public class ConnectorObjectStorage implements EntaxyObjectStorage {

    private static final Logger log = LoggerFactory.getLogger(ConnectorObjectStorage.class);

    @Reference(cardinality = ReferenceCardinality.MANDATORY, collectionType = CollectionType.SERVICE)
    protected ConnectorService connectorService;

    @Reference(cardinality = ReferenceCardinality.MANDATORY, collectionType = CollectionType.SERVICE)
    protected SystemService systemService;

    @SuppressWarnings("serial")
    protected Map<String, String> fieldMap = new HashMap<>() {{
        put("name", "objectId");
        put("version", "artifact.version");
    }};

    protected Map<String, String> fieldBundleMap = new HashMap<>() {{
        put("name", "bundleId");
        put("type", "artifact.type");
        put("status", "artifact.status");
        put("version", "artifact.version");
        put("url", "bundleLocation");
    }};

    @Override
    public String[] getObjectTypes() {
        return new String[]{EntaxyObject.OBJECT_TYPES.CONNECTOR};
    }

    @Override
    public Object store(Map<String, Object> data) {

        JsonObject jsonObject = new JsonObject();
        Gson gson = new Gson();

        for (Map.Entry<String, String> entry : fieldMap.entrySet()) {
            if (data.containsKey(entry.getValue()))
                jsonObject.add(entry.getKey(), gson.toJsonTree(data.get(entry.getValue())));
        }
        JsonObject bundleJsonObject = new JsonObject();
        for (Map.Entry<String, String> entry : fieldBundleMap.entrySet()) {
            if (data.containsKey(entry.getValue()))
                bundleJsonObject.add(entry.getKey(), gson.toJsonTree(data.get(entry.getValue())));
        }

        log.debug("JSON :: " + jsonObject.toString());
        ru.entaxy.esb.system.connector.entity.Connector connector = gson.fromJson(jsonObject, ru.entaxy.esb.system.connector.entity.Connector.class);
        connector.setTemplateName(data.get("factoryId").toString());
        connector.setCreateDate(new Date());
        connector.setCreatedBy("admin");

        log.debug("JSON Bundle :: " + bundleJsonObject.toString());
        BundleEntity bundleEntity = gson.fromJson(bundleJsonObject, BundleEntity.class);
        connector.setBundleEntity(bundleEntity);
        ru.entaxy.esb.system.jpa.entity.System system = systemService.getByUuid(data.get("profile").toString());
        connectorService.addNewConnector(system, connector);

        return null;
    }
}
