<?xml version="1.0" encoding="UTF-8"?>
<!--
  ~~~~~~licensing~~~~~~
  profile-producing
  ==========
  Copyright (C) 2020 - 2023 EmDev LLC
  ==========
  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at
  
       http://www.apache.org/licenses/LICENSE-2.0
  
  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
  ~~~~~~/licensing~~~~~~
  -->

<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:fn="http://www.w3.org/2005/xpath-functions" xmlns:local="http://ru.entaxy">
	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes" />
	
	<xsl:param name="profileName" select="'common'" />
	
	<xsl:template match="/">
		<xsl:variable name="processed">
			<xsl:sequence select="local:add-ids(./element()[1])" />
		</xsl:variable>
		<fragment>
			<xsl:for-each select="$processed//route//connector-selector">
				<xsl:call-template name="create-selector-bean" />
			</xsl:for-each>	
			<xsl:apply-templates select="$processed/element()[1]" />
		</fragment>
	</xsl:template>

	<xsl:function name="local:add-ids" as="element()*">
		<xsl:param name="root" />
		<xsl:apply-templates select="$root" />
	</xsl:function>	

	<xsl:template match="//connector-selector[not(@*)]" exclude-result-prefixes="xsl fn xs local">
		<connector-selector>
			<xsl:attribute name="id" select="replace(generate-id(), ':', '') " />
			<xsl:apply-templates select="child::node()" />
		</connector-selector>
	</xsl:template>
	
	<xsl:template match="//connector-selector[@id]" exclude-result-prefixes="xsl fn xs local">
		<xsl:comment>generated from 'connector-selector'</xsl:comment>
		<to>
			<xsl:attribute name="uri">
				<xsl:text>bean:selector-</xsl:text><xsl:value-of select="./@id" />
			</xsl:attribute> <!-- select="concat('bean',':','selector-', ./@id)" /-->
		</to>
	</xsl:template>	
	
	<xsl:template name="create-selector-bean">
		<bean class="ru.entaxy.platform.profiles.runtime.support.ConnectorSelector" activation="eager">
			<xsl:attribute name="id">
				<xsl:text>selector-</xsl:text><xsl:value-of select="./@id" />
			</xsl:attribute>
			<!-- select="concat('selector-', ./@id)" / -->
			<property name="config">
				<xsl:attribute name="value" select="local:generate-config(.)" />
			</property>
			<property name="profile">
				<xsl:attribute name="ref" select="concat('profile-', $profileName)" />
			</property>
		</bean>
	</xsl:template>

	<xsl:function name="local:generate-config">
		<xsl:param name="selector-node" />
		<xsl:text>{</xsl:text><xsl:apply-templates select="$selector-node/node()" /><xsl:text>"_end":true}</xsl:text>
	</xsl:function>

	<xsl:template match="//route[not(@id)]" exclude-result-prefixes="xsl fn xs">
		<route>
			<xsl:attribute name="id" select="concat('profile-', $profileName, '-connector-selector')" />
			<from>
				<xsl:attribute name="uri" select="concat('direct:profile-', $profileName, '-connector-selector')" />
			</from>
			<xsl:apply-templates select="./node()" />
		</route>
	</xsl:template>	

	<xsl:template match="//connector-selector[@id]/properties-to-use">
		<xsl:text>"properties":[</xsl:text>
		<xsl:for-each select="./property">
			<xsl:text>"</xsl:text><xsl:value-of select="./text()" /><xsl:text>"</xsl:text>
			<xsl:if test="./following-sibling::property"><xsl:text>,</xsl:text></xsl:if>
		</xsl:for-each>
		<xsl:text>],</xsl:text>
	</xsl:template>

	<xsl:template match="//connector-selector[@id]/patterns">
		<xsl:text>"patterns":[</xsl:text>
		<xsl:for-each select="./pattern">
			<xsl:text>"</xsl:text><xsl:value-of select="./text()" /><xsl:text>"</xsl:text>
			<xsl:if test="./following-sibling::pattern"><xsl:text>,</xsl:text></xsl:if>
		</xsl:for-each>
		<xsl:text>],</xsl:text>
	</xsl:template>

	<xsl:template match="//connector-selector[@id]/options">
		<xsl:text>"options":{</xsl:text>
		<xsl:for-each select="./element()">
			<xsl:text>"</xsl:text><xsl:value-of select="node-name(.)" /><xsl:text>":</xsl:text>
			<xsl:choose>
				<xsl:when test="string-length(./text()) > 0">
					<xsl:variable name="value" select="./text()" />
					<xsl:choose>
						<xsl:when test="($value = 'true') or ($value = 'false')">
							<xsl:value-of select="./text()" /><xsl:text>,</xsl:text>
						</xsl:when>
						<xsl:otherwise>
							<xsl:text>"</xsl:text><xsl:value-of select="./text()" /><xsl:text>",</xsl:text>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:when>
				<xsl:otherwise>
					<xsl:text>true,</xsl:text>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
		<xsl:text>"_end":true},</xsl:text>
	</xsl:template>

	<xsl:template name="default-copy" match="node()|@*">
		<xsl:copy>
			<xsl:apply-templates select="node()|@*" />
		</xsl:copy>
	</xsl:template>
		
</xsl:stylesheet>
