<?xml version="1.0" encoding="UTF-8"?>
<!--
  ~~~~~~licensing~~~~~~
  route-producing
  ==========
  Copyright (C) 2020 - 2023 EmDev LLC
  ==========
  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at
  
       http://www.apache.org/licenses/LICENSE-2.0
  
  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
  ~~~~~~/licensing~~~~~~
  -->

<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:fn="http://www.w3.org/2005/xpath-functions">
	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes" />
	
	<!-- xsl:param name="profileName" select="'common'" / -->
	
	<xsl:template match="/">
		<xsl:apply-templates select="child::node()" />
	</xsl:template>

	<xsl:template match="//set-destination" exclude-result-prefixes="xsl fn xs">
		<xsl:variable name="full" select="./text()" />
		<xsl:variable name="name">
			<xsl:choose>
				<xsl:when test="contains($full, ':')"><xsl:value-of select="substring-before($full, ':')" /></xsl:when>
				<xsl:otherwise><xsl:value-of select="$full" /></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="type" select="substring-after($full, ':')" />
		<xsl:comment>generated from 'set-destination'</xsl:comment>
		<setHeader name="ENTAXY_Destination">
			<constant><xsl:value-of select="$name" /></constant>
		</setHeader>
		<xsl:if test="string-length($type) > 0">
		<setHeader name="ENTAXY_DestinationType">
			<constant><xsl:value-of select="$type" /></constant>
		</setHeader>
		</xsl:if>
	</xsl:template>	

	<xsl:template match="//default-destination" exclude-result-prefixes="xsl fn xs">
		<xsl:variable name="full" select="./text()" />
		<xsl:variable name="name">
			<xsl:choose>
				<xsl:when test="contains($full, ':')"><xsl:value-of select="substring-before($full, ':')" /></xsl:when>
				<xsl:otherwise><xsl:value-of select="$full" /></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="type" select="substring-after($full, ':')" />
		<xsl:comment>generated from 'default-destination'</xsl:comment>
		<choice>
			<when>
				<simple>${headers.ENTAXY_Destination} == null</simple>
				<removeHeader headerName="ENTAXY_DestinationType" />
				<setHeader name="ENTAXY_Destination">
					<constant><xsl:value-of select="$name" /></constant>
				</setHeader>
				<xsl:if test="string-length($type) > 0">
					<setHeader name="ENTAXY_DestinationType">
						<constant><xsl:value-of select="$type" /></constant>
					</setHeader>
				</xsl:if>
			</when>
		</choice>		
	</xsl:template>	

	<xsl:template match="//next-route" exclude-result-prefixes="xsl fn xs">
		<xsl:variable name="local">
			<xsl:choose>
				<xsl:when test="./@local">
					<xsl:value-of select="./@local" />
				</xsl:when>
				<xsl:otherwise><xsl:text>false</xsl:text></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>

		<xsl:variable name="async">
			<xsl:choose>
				<xsl:when test="./@async">
					<xsl:value-of select="./@async" />
				</xsl:when>
				<xsl:otherwise><xsl:text>false</xsl:text></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		
		<xsl:variable name="component">
			<xsl:choose>
				<xsl:when test="$async = 'false'">
					<xsl:text>direct</xsl:text>
					<xsl:if test="$local = 'false'"><xsl:text>-vm</xsl:text></xsl:if>
				</xsl:when>
				<xsl:otherwise>
					<xsl:choose>
						<xsl:when test="$local = 'true'"><xsl:text>seda</xsl:text></xsl:when>
						<xsl:otherwise><xsl:text>vm</xsl:text></xsl:otherwise>
					</xsl:choose>	
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>	
		
		<xsl:variable name="name">
			<xsl:value-of select="./text()" />
		</xsl:variable>
		<xsl:comment>generated from 'next-route'</xsl:comment>
		<setHeader name="NTX_NextPoint">
			<constant><xsl:value-of select="$component" /><xsl:text>:</xsl:text><xsl:value-of select="$name" /></constant>
		</setHeader>
	</xsl:template>	


	<xsl:template name="default-copy" match="node()|@*">
		<xsl:copy>
			<xsl:apply-templates select="node()|@*" />
		</xsl:copy>
	</xsl:template>
		
</xsl:stylesheet>
