/*-
 * ~~~~~~licensing~~~~~~
 * entaxy-management-plugin
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
var Entaxy;
(function (Entaxy) {
    Entaxy._module.component('entaxyAddConnectorModal', {
        bindings: {
            modalInstance: '<',
            resolve: '<'
        },
        template:
        `
        <div class="entaxy-modal-container">
          <div class="modal-header">
            <button type="button" class="close" aria-label="Close" ng-click="$ctrl.cancel()">
              <span class="pficon pficon-close" aria-hidden="true"></span>
            </button>
            <h4 class="modal-title">Add Connector to {{$ctrl.profileName}}</h4>
          </div>
          <div ng-if="$ctrl.step1">
            <div class="modal-body-header">
              <h2>Choose connector template</h2>
            </div>
            <div class="modal-body" style="height:433px; max-height:433px">
                <entaxy-modal-list items="$ctrl.viewedTemplates" selected="$ctrl.selectedItem"
                      change-selection="$ctrl.changeSelection"></entaxy-modal-list>
            </div>
            <div class="modal-footer">
              <button type="button" class="btn btn-default" ng-click="$ctrl.next()"
                  ng-disabled="!$ctrl.selectedItem">Next</button>
            </div>
          </div>
          <form name="connectorForm" class="form-horizontal" ng-if="$ctrl.step2" ng-submit="$ctrl.saveConnector($ctrl.formFields)">
            <div class="modal-body" style="height:503px; max-height:503px;">
              <div class="modal-body-header">
                <h2>Fill in connector template parameters</h2>
              </div>
              <div class="form-group" ng-class="{'has-error': $ctrl.errors[formField.name]}"
                    ng-repeat="formField in $ctrl.formFields" style="margin:15px;">
                <label class="col-sm-3 control-label" ng-class="{'required-pf': formField.isRequired}" for="{{formField.name}}">{{formField.label}}</label>
                <div class="col-sm-8">
                  <input type="{{formField.type}}" id="{{formField.name}}" ng-class="{'form-control': formField.type !== 'checkbox'}"
                        ng-model="formField.value" ng-readonly="formField.isReadOnly">
                  <span class="help-block" ng-show="$ctrl.errors[formField.name]">{{$ctrl.errors[formField.name]}}</span>
                </div>
              </div>
            </div>
            <div class="modal-footer">
              <button type="button" class="btn btn-default" ng-click="$ctrl.back()">Back</button>
              <button type="submit" class="btn btn-primary">Add</button>
            </div>
          </form>
        </div>
        `,
        controller: EntaxyAddConnectorModalController
    })
    .name;

    function EntaxyAddConnectorModalController(workspace) {
        'ngInject';

        var ctrl = this;

        ctrl.$onInit = function() {
            ctrl.step1 = true;
            ctrl.step2 = false;

            let templates = Entaxy.getConnectorTemplates();
            ctrl.templates = templates.map(template => { return { name: template, displayName: template }});
            ctrl.viewedTemplates = ctrl.templates;

            ctrl.profileName = ctrl.resolve.profileName;
        }

        ctrl.listConfig = {
            selectionMatchProp: 'name',
            selectItems: true,
            showSelectBox: false,
            dlbClick: true,
            onDblClick: function(item, event){
            	ctrl.listConfig.selectedItems.push(item);
            	ctrl.next();
            }
        }

        ctrl.changeSelection = function(item, isDblClicked) {
            ctrl.selectedItem = item;
            if (isDblClicked) {
                ctrl.next();
            }
        }

        ctrl.next = function() {
            ctrl.step1 = false;
            ctrl.step2 = true;

            if (!ctrl.formFields || ctrl.selectedItem.name !== ctrl.formFields[0].value) {
                ctrl.formFields = Entaxy.getConnectorTemplateParams(ctrl.selectedItem.name);
                ctrl.formFields.unshift({
                    label: 'templateName',
                    displayName: 'Template Name',
                    type: 'java.lang.String',
                    helpText: null,
                    value: ctrl.selectedItem.name,
                    isReadOnly: true
                });
                ctrl.formFields.forEach(formField => {
                    formField.name = formField.label;
                    if (formField.displayName) {
                        formField.label = formField.displayName;
                    }
                    formField.type = Entaxy.convertToHtmlInputType(formField.type);
                });
            }
        }

        ctrl.back = function() {
            ctrl.step1 = true;
            ctrl.step2 = false;
        }

        ctrl.cancel = function() {
            ctrl.modalInstance.dismiss();
        }

        ctrl.saveConnector = function(fields) {
            ctrl.errors = validateFields(fields);

            if (Object.keys(ctrl.errors).length === 0) {
                let connectorArgs = {
                    connectorTemplateName: ctrl.formFields.shift().value,
                    connectorFields: ctrl.formFields.map(formField => { return { label: formField.name, value: formField.value }; })
                };

                ctrl.modalInstance.close(connectorArgs);
            }
        }

        function validateFields(fields) {
            let errors = {};

            _.forEach(fields, (field) => {
                if (field.isRequired && (!field.value || (field.type !== 'checkbox' ? field.value.toString().trim().length === 0 : false))) {
                    errors[field.name] = 'Please fill out this field';
                }
            });

            return errors;
        }
    }
    EntaxyAddConnectorModalController.$inject = ['workspace'];

})(Entaxy || (Entaxy = {}));
