/*-
 * ~~~~~~licensing~~~~~~
 * entaxy-management-plugin
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
var Entaxy;
(function (Entaxy) {
    Entaxy._module
    .component('entaxyConnectors', {
        bindings: {
            pageTitle: '@'
        },
        template:
        `
          <h2>
            {{$ctrl.pageTitle}}
          </h2>

          <entaxy-table item-name="Connector" folder-name="Коннекторы"
                status-name="BundleState" object-name="systemName"
                extra-primary-actions-config="$ctrl.config"
                is-parent-dependent="true"></entaxy-table>

        `,
        controller: EntaxyConnectorsController
    })
    .name;

    function EntaxyConnectorsController(workspace, $scope, $uibModal, operationsService, jolokiaService) {
        'ngInject';

        var ctrl = this;
        ctrl.workspace = workspace;
        var entaxyJmxDomain = localStorage['entaxyJmxDomain'] || "ru.entaxy.esb";

        let primaryActions = [ { name: 'Add Legacy Connector', actionFn: () => ctrl.showAddConnectorModal() } ];
        ctrl.config = {
            primaryActions: primaryActions,
            isTableUpdateNeeded: true
        }

        ctrl.connectors = [];
        ctrl.viewedConnectors = [];

        ctrl.showAddConnectorModal = function() {
            let selectedMBean = workspace.getSelectedMBean();

            $uibModal.open({
                component: 'entaxyAddConnectorModal',
                resolve: {
                    connector: null,
                    profileName: () => selectedMBean.title
                },
                size: 'lg',
                backdrop: 'static'
            })
            .result.then(connectorArgs => {
                addConnector(connectorArgs);
            });
        }

        function addConnector(connectorArgs) {

            let args = [];
            args.push(connectorArgs.connectorTemplateName);
            let argParams = '{';
            _.forEach(connectorArgs.connectorFields, (field) => {
                argParams += '\"' + field.label + '\":\"' + field.value + '\",';
            });
            if (argParams.length > 1) {
                argParams = argParams.substring(0, argParams.length - 1);
            }
            argParams += '}';
            args.push(argParams);

            let mbeanName = ctrl.workspace.getSelectedMBeanName();

            operationsService.executeOperation(mbeanName, { name: 'addConnector' }, args)
            .then(result => {
                if (ctrl.config.isTableUpdateNeeded && ctrl.config.updateTableFn) {
                    $scope.$on(Jmx.TreeEvent.Updated, ctrl.config.updateTableFn);
                }

                Core.notification('success', result, 3000);
            }).catch(error => {
                Core.notification('danger', error, 5000);
            });
        }

        ctrl.dismissAlert = () => ctrl.alert = null;

    }
    EntaxyConnectorsController.$inject = ['workspace', '$scope', '$uibModal', 'operationsService', 'jolokiaService'];

})(Entaxy || (Entaxy = {}));
