/*-
 * ~~~~~~licensing~~~~~~
 * entaxy-management-plugin
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
var Entaxy;
(function (Entaxy) {
    Entaxy._module.component('entaxyModal', {
        bindings: {
            modalInstance: '<',
            resolve: '<'
        },
        template:
        `
          <div class="entaxy-modal-container">
            <div class="modal-header">
              <button type="button" class="close" aria-label="Close" ng-click="$ctrl.cancel()">
                <span class="pficon pficon-close" aria-hidden="true"></span>
              </button>
              <h4 class="modal-title">{{$ctrl.modalTitle}}</h4>
            </div>

            <div ng-if="$ctrl.step1">
              <div class="modal-body-header">
                <h2>{{$ctrl.firstStepTitle}}</h2>
              </div>
              <div class="modal-body">
                <entaxy-modal-list-with-description items="$ctrl.factories" selected-item="$ctrl.selectedItem"
                    change-selection="$ctrl.changeSelection"></entaxy-modal-list-with-description>
              </div>
              <div class="modal-footer">
                <button type="button" class="btn btn-default" ng-click="$ctrl.next()"
                      ng-disabled="!$ctrl.selectedItem">Next</button>
              </div>
            </div>

            <div ng-if="$ctrl.step2">
              <div class="modal-body-header">
                <h2>{{$ctrl.secondStepTitle}}</h2>
              </div>
              <div class="modal-body">
                <entaxy-modal-group-fields groups="$ctrl.groups" fields="$ctrl.formFields"
                      errors="$ctrl.errors" mode="$ctrl.mode"></entaxy-modal-group-fields>
              </div>
              <div class="modal-footer">
                <button type="button" class="btn btn-default" ng-if="$ctrl.hasSteps" ng-click="$ctrl.back()">Back</button>
                <button type="submit" class="btn btn-primary" ng-click="$ctrl.save($ctrl.formFields)">{{$ctrl.submitBtnTitle}}</button>
              </div>
            <div>
          </div>
        `,
        controller: entaxyModalController
    })
    .name;

    function entaxyModalController(operationsService, entaxyService) {
        'ngInject';

        var ctrl = this;

        ctrl.$onInit = function() {

            ctrl.mode = ctrl.resolve.mode;

            if (!ctrl.mode) {
                ctrl.cancel('Mode is not defined');
            }

            ctrl.itemType = ctrl.resolve.itemType ? ctrl.resolve.itemType : 'Item';

            let extraToTitle = ctrl.resolve.parentName ? (' to ' + ctrl.resolve.parentName) : '';

            ctrl.modalTitle = ctrl.mode + ' ' + ctrl.itemType + extraToTitle;

            ctrl.submitBtnTitle = Entaxy.getButtonTitleByMode(ctrl.mode);

            let itemName = ctrl.resolve.itemName;

            ctrl.firstStepTitle = ctrl.resolve.firstStepTitle ? ctrl.resolve.firstStepTitle : 'Choose factory';

            ctrl.secondStepTitle = itemName ? ctrl.mode + ' properties for ' + itemName
                : ctrl.resolve.secondStepTitle ? ctrl.resolve.secondStepTitle : 'Fill in fields';

            if (ctrl.mode !== Entaxy.MODAL_MODES.ADD) {
                let properties = Object.entries(ctrl.resolve.info.properties);
                ctrl.properties = properties.map(property => { return { name: property[0], value: property[1] }; });
            }

            ctrl.factories = ctrl.resolve.factories ? ctrl.resolve.factories : null;

            if (ctrl.factories && ctrl.factories.length !== 0) {
                if (ctrl.factories.length > 1) {
                    ctrl.factories.sort(Entaxy.compareBy('displayName'));
                    ctrl.step1 = true;
                    ctrl.step2 = false;
                    ctrl.hasSteps = true;
                } else if (ctrl.factories.length === 1) {
                    ctrl.hasSteps = false;
                    ctrl.next();
                }
            } else {
                ctrl.cancel('Factories are not found');
            }
        }

        ctrl.next = function() {

            if (!ctrl.formFields || ctrl.selectedItem.name !== ctrl.formFields[0].value) {

                ctrl.formFields = [];

                ctrl.formFields.push({
                    label: 'Factory Id',
                    name: 'factoryId',
                    type: 'java.lang.String',
                    helpText: null,
                    value: ctrl.hasSteps ? ctrl.selectedItem.name : ctrl.factories[0].name,
                    readOnly: true,
                    required: ctrl.mode === Entaxy.MODAL_MODES.VIEW ? false : true,
                    group: 'general'
                });

                let mbeanName = ctrl.hasSteps ? ctrl.selectedItem.mbeanName : ctrl.factories[0].mbeanName;

                operationsService.executeOperation(mbeanName, { name: 'getFields' }, [ 'init' ])
                .then((response) => {

                    let groups = new Set();
                    groups.add('general');

                    _.forEach(JSON.parse(response), (field) => {

                        let objectId = undefined;
                        if (ctrl.mode !== Entaxy.MODAL_MODES.ADD) {
                            objectId = ctrl.resolve.info['objectId'];
                        }
                        let formField = entaxyService.makeFormField(field, objectId, ctrl.properties, ctrl.mode);

                        ctrl.formFields.push(formField);
                        groups.add(formField.group);
                    });

                    ctrl.formFields.forEach(formField => formField.type = Entaxy.convertToHtmlInputType(formField.type));

                    ctrl.groups = Array.from(groups).map((group) => { return { name: group, displayName: group }; });

                    ctrl.step1 = false;
                    ctrl.step2 = true;
                });
            } else {
                ctrl.step1 = false;
                ctrl.step2 = true;
            }
        }

        ctrl.back = function() {
            ctrl.step1 = true;
            ctrl.step2 = false;
        }

        ctrl.cancel = function(reason) {
            ctrl.modalInstance.dismiss(reason);
        }

        ctrl.changeSelection = function(item, isDblClicked) {
            ctrl.selectedItem = item;
            if (isDblClicked) {
                ctrl.next();
            }
        }

        ctrl.save = function(fields) {

            if (ctrl.mode === Entaxy.MODAL_MODES.VIEW) {
                ctrl.cancel();
            }

            let selectedFactory = ctrl.selectedItem ? ctrl.selectedItem : ctrl.factories[0];

            entaxyService.validateFields(fields, selectedFactory, ctrl.itemType)
            .then(errors => {
                ctrl.errors = errors;

                if (Object.keys(ctrl.errors).length === 0) {
                    let args = entaxyService.getArguments(fields, ctrl.factories);
                    ctrl.modalInstance.close(args);
                }
            });
        }
    }
    entaxyModalController.$inject = ['operationsService', 'entaxyService'];

})(Entaxy || (Entaxy = {}));
