/*-
 * ~~~~~~licensing~~~~~~
 * entaxy-management-plugin
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
var Entaxy;
(function (Entaxy) {
    Entaxy._module
    .component('entaxyModalGroupFields', {
        bindings: {
            groups: '<',
            fields: '<',
            errors: '<',
            mode: '<'
        },
        template:
        `
          <div class="modal-group-fields-container">
            <div class="modal-groups-list-container">
              <entaxy-modal-list class="group-list" items="$ctrl.groups" items-label="Group" items-label-plural="Groups"
                selected="$ctrl.selectedGroup" change-selection="$ctrl.change" filter="$ctrl.matchesFilter"></entaxy-modal-list>
            </div>
            <div class="modal-group-fields-form-container">
              <form name="connectorForm" class="form-horizontal">
                <div class="form-group" ng-class="{'has-error': $ctrl.errors[formField.name]}"
                      ng-repeat="formField in $ctrl.viewedFormFields" ng-if="!formField.isBackRef && !formField.isHidden">
                  <div class="col-sm-4 label-col">
                    <label class="control-label" ng-class="{'required-pf': formField.required}" for="{{formField.name}}">{{formField.label}}</label>
                    <button type="button" class="btn btn-link label-description-popover" popover-placement="auto top-left" popover-trigger="'outsideClick'"
                          uib-popover="{{formField.description}}" ng-if="formField.description">
                      <span class="pficon pficon-help"></span>
                    </button>
                  </div>
                  <div class="col-sm-7">
                    <input type="{{formField.type}}" id="{{formField.name}}" placeholder="{{formField.placeholder}}" ng-class="{'form-control': formField.type !== 'checkbox'}"
                      ng-model="formField.value" ng-readonly="formField.readOnly" ng-disabled="formField.type === 'checkbox' && formField.readOnly"
                      ng-if="!formField.type.startsWith('entaxy.runtime.') && formField.type !== 'xml:route' && !formField.typeInfo">
                    <entaxy-select type="formField.type" id="{{formField.name}}" filter="formField.filter" model="formField.value"
                      update-parent-fn="$ctrl.updateFieldsAfterSelectionChange" options="formField.options" ng-readonly="formField.readOnly"
                      ng-if="formField.type.startsWith('entaxy.runtime.')"></entaxy-select>
                    <entaxy-select-from-enum id="{{formField.name}}" values="formField.typeInfo.values" model="formField.value"
                      ng-readonly="formField.readOnly" ng-if="formField.typeInfo && formField.typeInfo.type === 'enum'"></entaxy-select-from-enum>
                    <entaxy-xml id="{{formField.name}}" ng-model="formField.value" ng-if="formField.type === 'xml:route'" mode="$ctrl.localMode"></entaxy-xml>
                    <span class="help-block" ng-show="$ctrl.errors[formField.name]">{{$ctrl.errors[formField.name]}}</span>
                  </div>
                </div>
              </form>
            </div>
          </div>
        `,
        controller: entaxyModalGroupFieldsController
    })
    .name;

    function entaxyModalGroupFieldsController(operationsService) {
        'ngInject';

        var ctrl = this;

        ctrl.$onInit = function() {
            ctrl.localMode = ctrl.mode === Entaxy.MODAL_MODES.ADD ? Entaxy.MODAL_MODES.EDIT : ctrl.mode;

            ctrl.selectedGroup = ctrl.groups[0];
        }

        ctrl.$onChanges = function(changes) {
            if (changes.errors && changes.errors.currentValue) {
                let errorKeys = Object.keys(changes.errors.currentValue);
                if (errorKeys.length > 0) {
                    let errorField = ctrl.fields.find(field => field.name === errorKeys[0]);
                    ctrl.selectedGroup = ctrl.groups.find(group => group.name === errorField.group);
                }
            }
        }

        ctrl.change = function(group) {
            ctrl.viewedFormFields = ctrl.fields.filter((formField) => formField.group === group.name);
            ctrl.selectedGroup = group;
        }

        ctrl.matchesFilter = function(group, filter) {
            var match = false;

            _.forEach(ctrl.fields.filter((formField) => formField.group === group.name), (field) => {
                if (field.tag && field.tag.length !== 0) {
                    if (field.tag.toLowerCase().match(filter.value.toLowerCase()) !== null) {
                        match = true;
                        return match;
                    }
                }
            });
            match = group.displayName.toLowerCase().match(filter.value.toLowerCase()) !== null;

            return match;
        };

        ctrl.updateFieldsAfterSelectionChange = function (oldItem, newItem, itemType) {

            if (oldItem) {
                changeFieldsFromSelected(oldItem, itemType, true);
            }

            changeFieldsFromSelected(newItem, itemType, false);
        }

        function changeFieldsFromSelected(item, itemType, toDefaults) {
            if (item.mbeanName) {
                operationsService
                .executeOperation(item.mbeanName, { name: 'get' + itemType + 'Config' }, [ item.name ])
                .then(result => {
                    let itemInfo = JSON.parse(result);
                    let properties = itemInfo.properties;

                    changeFieldsValuesFromProperties(properties, toDefaults);

                }).catch(error => {
                    Core.notification('danger', error, 5000);
                    Entaxy.log.error(error);
                });
            } else if (item.value) {
                let properties = item.value.properties;

                changeFieldsValuesFromProperties(properties, toDefaults);
            }
        }

        function changeFieldsValuesFromProperties(properties, toDefaults) {
            if (properties && Object.keys(properties).length > 0) {
                let keys = Object.keys(properties);

                keys.forEach(key => {
                    let field = ctrl.fields.find(field => field.name === key);
                    if (field && !field.isHidden) {
                        field.value = toDefaults ? field.defaultValue : properties[key];
                    }
                });
            }
        }
    }
    entaxyModalGroupFieldsController.$inject = ['operationsService'];

})(Entaxy || (Entaxy = {}));
