/*-
 * ~~~~~~licensing~~~~~~
 * entaxy-management-plugin
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
var Entaxy;
(function (Entaxy) {
    // fixme this component works only for connections now
    Entaxy._module
    .component('entaxySelect', {
        bindings: {
            type: '<',
            filter: '<',
            model: '=',
            updateParentFn: '<',
            options: '='
        },
        template:
        `
          <div class="custom-select">
            <button type="button" class="btn-select form-control dropdown-toggle" id="filter" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
              {{$ctrl.selectedOption.name}}
              <span class="caret"></span>
            </button>
            <ul class="dropdown-menu">
              <li ng-repeat="option in $ctrl.options" id="option.name" ng-click="$ctrl.changeSelection(option)">{{ option.name }}</li>
              <li id="addItem" class="dropdown-option-new" ng-click="$ctrl.addConnection()"><span class="pficon pficon-add-circle-o"></span>New private connection...</li>
            </ul>
          </div>
        `,
        controller: entaxySelectController
    })
    .name;

    function entaxySelectController(workspace, jolokiaService, $q, $uibModal) {
        'ngInject';

        var ctrl = this;

        ctrl.$onInit = function() {

            if (ctrl.type === 'entaxy.runtime.connection') {

                if (!ctrl.options) {

                    let promises = [];
                    let filteredConnections = [];

                    let connectionsFolder = workspace.tree.findDescendant(child => (child.objectName && child.objectName.endsWith('category=connections')));

                    if (connectionsFolder) {
                        if (ctrl.filter) {
                            connectionsFolder.children.forEach((child) => {
                                promises.push(
                                    jolokiaService.getAttribute(child.objectName, 'Label')
                                    .then((label) => {
                                        if (label.indexOf(ctrl.filter) > -1) {
                                            filteredConnections.push({ name: child.title, mbeanName: connectionsFolder.objectName });
                                        }
                                    })
                                );
                            });
                        } else {
                            filteredConnections = connectionsFolder.children ? connectionsFolder.children.map(child => { name: child.title }) : [];
                        }
                    }

                    $q.all(promises).then(() => {
                        if (filteredConnections && filteredConnections.length > 0) {
                            ctrl.options = filteredConnections.sort(Entaxy.compareBy('name'));
                            ctrl.changeSelection(filteredConnections[0]);
                        } else {
                            Core.notification('danger', 'There are no suitable connections', 5000);
                            // todo close the modal
                        }
                    });
                } else {
                    ctrl.selectedOption = ctrl.options.find((option) => option.name === ctrl.model || option.value === ctrl.model);
                }
            }
        }

        ctrl.changeSelection = function (option) {
            let previousOption = ctrl.selectedOption;
            ctrl.selectedOption = option;
            ctrl.model = option.value ? option.value : option.name;

            if (ctrl.type === 'entaxy.runtime.connection') {
                if (previousOption) {
                    Core.notification('warning', 'Changing connection field value may lead to changes in other fields of the current connector. ' +
                        'Please, make sure to revise all fields before accepting.', 10000);
                }
                ctrl.updateParentFn(previousOption, option, 'Connection');
            }
        }

        ctrl.addConnection = function () {
            ctrl.showModal();
        }

        ctrl.showModal = function() {

            let factories = [];
            let promises = [];

            let factoryFolder = workspace.tree.findDescendant(child => child.title === ctrl.type);

            if (factoryFolder && factoryFolder.children) {
                factoryFolder.children.forEach((child) => {
                    promises.push(jolokiaService.getAttributes(child.objectName, ['Abstract', 'DisplayName', 'Label', 'Description', 'Deprecated'])
                    .then((result) => {
                        let label = result['Label'];

                        if (!result['Abstract']) {
                            if (!ctrl.filter || (ctrl.filter && label && label.indexOf(ctrl.filter) > -1)) {
                                factories.push({
                                    name: child.title,
                                    displayName: result['DisplayName'] ? result['DisplayName'] : child.title,
                                    label: result['Label'],
                                    mbeanName: child.objectName,
                                    description: result['Description'] ? result['Description'] : 'There is no description for this factory.',
                                    additionalInfo: result['Deprecated'] ? ['[DEPRECATED]'] : undefined
                                });
                            }
                        }
                    }));
                });

                $uibModal.open({
                    component: 'entaxyModal',
                    resolve: {
                        mode: () => Entaxy.MODAL_MODES.ADD,
                        itemType: () => 'Connection',
                        factories: $q.all(promises).then(() => factories)
                    },
                    size: 'xl',
                    backdrop: 'static',
                    windowTopClass: 'modal-top-margin-override'
                })
                .result.then(args => {
                    addItem(args);
                },
                reason => {
                    if (reason) {
                        Core.notification('danger', reason, 5000);
                    }
                });
            }
        }

        function addItem(args) {

            let factoryId = args.factoryId.name;

            let objectId = args.fields.find((field) => field.name === 'objectId').value;

            let fields = objectId ? args.fields.filter((field) => field.name !== 'objectId') : args.fields;

            let properties = fields.reduce((obj, cur) => ({ ...obj, [cur.name] : cur.value }), {});

            let newObjectProperties = {
                factoryId: factoryId,
                objectId: objectId,
                scope: 'private',
                properties: properties
            };

            let newObjectOption = {
                name: objectId ? objectId : 'private-connection',
                value: newObjectProperties
            };

            ctrl.options.push(newObjectOption);
            ctrl.changeSelection(newObjectOption);
        }
    }
    entaxySelectController.$inject = ['workspace', 'jolokiaService', '$q', '$uibModal'];

})(Entaxy || (Entaxy = {}));
