/*-
 * ~~~~~~licensing~~~~~~
 * entaxy-management-plugin
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
var Entaxy;
(function (Entaxy) {
    Entaxy._module
    .component('entaxySource', {
        template:
        `
          <div class="route-source-header-container">
            <h2>
              Source
            </h2>
            <span class="route-source-error-message" ng-if="$ctrl.errorMessage">*{{$ctrl.errorMessage}}</span>
          </div>
          <div ng-if="$ctrl.isContentPresent">
            <div class="xml-route-source-container">
              <div hawtio-editor="xml" mode="'xml'" output-editor="xmlEditor"></div>
            </div>
            <button type="submit" class="btn btn-primary" ng-click="$ctrl.save()">Save changes</button>
          </div>
          <div ng-if="$ctrl.isContentPresent === false">There is no viewable content.</div>
        `,
        controller: entaxySourceController
    })
    .name;

    function entaxySourceController(workspace, operationsService, $scope) {
        'ngInject';

        let ctrl = this;

        ctrl.$onInit = function() {

            let mbeanName = workspace.getSelectedMBeanName();

            operationsService
            .executeOperation(mbeanName, { name: 'doGetRouteConfig' }, [])
            .then(result => {
                let info = JSON.parse(result);

                let xml = info.properties.routeContent;
                if (xml) {
                    ctrl.isContentPresent = true;

                    ctrl.objectId = info.objectId;
                    ctrl.factoryId = info.factoryId;
                    ctrl.scope = info.scope;
                    ctrl.systemName = info.properties.systemName;

                    $scope.xml = atob(xml);
                } else {
                    ctrl.isContentPresent = false;
                }

            }).catch(error => {
                Core.notification('danger', error, 5000);
                Entaxy.log.error(error);
            });
        }

        ctrl.save = function() {
            if ($scope.xml.trim().length !== 0) {
                ctrl.errorMessage = '';

                let factoryFolder = workspace.tree.findDescendant(child => child.title === ctrl.factoryId);
                let mbeanName = factoryFolder.objectName
                let properties = { systemName: ctrl.systemName, routeContent: btoa($scope.xml) };
                let instructions = {"@LIFECYCLE":["general"]};

                operationsService.executeOperation(mbeanName, { name: 'createObjectByInstructions' }, [ ctrl.objectId, ctrl.scope, instructions, properties ] )
                .then(result => {
                    let tableHtml = Entaxy.createTableFromResponse(result);
                    Entaxy.notification('success', tableHtml, 10000);
                }).catch(error => {
                    Core.notification('danger', error, 5000);
                    Entaxy.log.error(error);
                });
            } else {
                ctrl.errorMessage = 'Value cannot be empty';
            }
        };

        setTimeout(function() {
            $scope.xmlEditor.options.extraKeys = Entaxy.getExtraKeys();
            $scope.xmlEditor.options.hintOptions = Entaxy.getHintOptionsForRouteXml();
        }, 100);
    }
    entaxySourceController.$inject = ['workspace', 'operationsService', '$scope'];

})(Entaxy || (Entaxy = {}));
