/*-
 * ~~~~~~licensing~~~~~~
 * entaxy-management-plugin
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
var Entaxy;
(function (Entaxy) {
    Entaxy._module.component('xmlModal', {
        bindings: {
            modalInstance: '<',
            resolve: '<'
        },
        template:
        `
          <div class="entaxy-modal-container">
            <div class="modal-header">
              <button type="button" class="close" aria-label="Close" ng-click="$ctrl.cancel()">
                <span class="pficon pficon-close" aria-hidden="true"></span>
              </button>
              <h4 class="modal-title">{{$ctrl.modalTitle}}</h4>
            </div>
              <div class="modal-body-without-header">
                <div class="xml-editor-container">
                  <div hawtio-editor="xml" mode="'xml'" output-editor="xmlEditor"></div>
                </div>
              </div>
              <div class="modal-footer">
                <button type="submit" class="btn btn-primary" ng-click="$ctrl.save($ctrl.formFields)">{{$ctrl.btnTitle}}</button>
              </div>
          </div>
        `,
        controller: xmlModalController
    })
    .name;

    function xmlModalController(workspace, $uibModal, $scope) {
        'ngInject';

        let ctrl = this;

        ctrl.$onInit = function() {
            ctrl.mode = ctrl.resolve.mode;
            ctrl.modalTitle = ctrl.mode + ' XML';
            ctrl.btnTitle = Entaxy.getButtonTitleByMode(ctrl.mode);

            $scope.xml = ctrl.resolve.xml;
        }

        ctrl.cancel = function(reason) {
            ctrl.modalInstance.dismiss(reason);
        }

        ctrl.save = function() {
            ctrl.modalInstance.close($scope.xml);
        }

        setTimeout(function() {
            if (ctrl.mode === Entaxy.MODAL_MODES.VIEW) {
                $scope.xmlEditor.options.readOnly = true;
            } else {
                $scope.xmlEditor.options.extraKeys = Entaxy.getExtraKeys();
                $scope.xmlEditor.options.hintOptions = Entaxy.getHintOptionsForRouteXml();
            }
        }, 100);
    }
    xmlModalController.$inject = ['workspace', '$uibModal', '$scope'];

})(Entaxy || (Entaxy = {}));
