/*-
 * ~~~~~~licensing~~~~~~
 * entaxy-management-plugin
 * ==========
 * Copyright (C) 2020 - 2023 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
var Entaxy;
(function (Entaxy) {

    let CodeMirror = window.CodeMirror;

    function completeAfter(cm, pred) {
        var cur = cm.getCursor();
        if (!pred || pred()) setTimeout(function() {
            if (!cm.state.completionActive) {
                cm.showHint({completeSingle: false});
            }
        }, 100);
        return CodeMirror.Pass;
    }

        function completeIfAfterLt(cm) {
            return completeAfter(cm, function() {
                var cur = cm.getCursor();
                return cm.getRange(CodeMirror.Pos(cur.line, cur.ch - 1), cur) == "<";
            });
        }

        function completeIfInTag(cm) {
            return completeAfter(cm, function() {
                var tok = cm.getTokenAt(cm.getCursor());
                if (tok.type == "string" && (!/['"]/.test(tok.string.charAt(tok.string.length - 1)) || tok.string.length == 1)) {
                    return false;
                }
                var inner = CodeMirror.innerMode(cm.getMode(), tok.state).state;
                return inner.tagName;
            });
        }


        let extraKeys = {
            "'<'": completeAfter,
            "' '": completeIfInTag,
            "'='": completeIfInTag,
            "':'": completeIfInTag,
            "Ctrl-Space": "autocomplete"
        };

    function getHintOptionsForRouteXml() {
        let tags = Entaxy.getCamelBlueprintTags();
        let hintOptions = { schemaInfo: tags };
        return hintOptions;
    }
    Entaxy.getHintOptionsForRouteXml = getHintOptionsForRouteXml;

    function getExtraKeys() {
        return extraKeys;
    }
    Entaxy.getExtraKeys = getExtraKeys;

})(Entaxy || (Entaxy = {}));
